// Asynch_IO_Impl.cpp,v 4.7 2002/04/16 22:47:26 shuston Exp

#include "ace/Asynch_IO_Impl.h"

#if (defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)) || (defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION))
// This only works on Win32 platforms and on Unix platforms supporting
// aio calls. 
#include "ace/OS.h"
#include "ace/Log_Msg.h"

#if !defined (__ACE_INLINE__)
#include "ace/Asynch_IO_Impl.i"
#endif /* __ACE_INLINE__ */

ACE_Asynch_Result_Impl::ACE_Asynch_Result_Impl (void)
  : act_              (0),
    completion_key_   (0),
    bytes_transferred_(0),
    error_            (0),
    state_            (INITIALIZED),
    op_code_          (NO_OP),
    flags_            (NORMAL_RESULT)
{
}

void
ACE_Asynch_Result_Impl::dump ()
{
  const ACE_TCHAR * str_state;
  const ACE_TCHAR * str_op;

  switch ( this->state_ )
    {
    case INITIALIZED:  str_state=ACE_LIB_TEXT("INITIALIZED "); break;
    case STARTED:      str_state=ACE_LIB_TEXT("STARTED     "); break;
    case START_ERROR:  str_state=ACE_LIB_TEXT("ERROR       "); break;
    case DEFERRED:     str_state=ACE_LIB_TEXT("DEFERRED    "); break;
    case RESULT_READY: str_state=ACE_LIB_TEXT("RESULT_READY"); break;
    case POSTED:       str_state=ACE_LIB_TEXT("POSTED      "); break;
    case COMPLETED:    str_state=ACE_LIB_TEXT("COMPLETED  "); break;
    default:           str_state=ACE_LIB_TEXT("?????????"); break;
    }

  switch ( this->op_code_ )
    {
    case NO_OP:        str_op=ACE_LIB_TEXT("NO"); break;
    case READ_STREAM:  str_op=ACE_LIB_TEXT("RS"); break;
    case WRITE_STREAM: str_op=ACE_LIB_TEXT("WS"); break;
    case READ_FILE:    str_op=ACE_LIB_TEXT("RF"); break;
    case WRITE_FILE:   str_op=ACE_LIB_TEXT("WF"); break;
    case ACCEPT:       str_op=ACE_LIB_TEXT("A "); break;
    case CONNECT:      str_op=ACE_LIB_TEXT("C "); break;
    case READ_DGRAM:   str_op=ACE_LIB_TEXT("RD"); break;
    case WRITE_DGRAM:  str_op=ACE_LIB_TEXT("WD"); break;
    case TRANSMIT_FILE:str_op=ACE_LIB_TEXT("TF"); break;
    case TIMER:        str_op=ACE_LIB_TEXT("TM"); break;
    default:           str_op=ACE_LIB_TEXT("??"); break;
    }


  ACE_DEBUG((LM_DEBUG,
             ACE_LIB_TEXT("op=%s state=%s error=%d count=%d\n"),
             str_op,
             str_state,
             this->error_,
             this->bytes_transferred_ ));
}

int
ACE_Asynch_Result_Impl::set_started ()
{
  //remove assert after tests
  //ACE_ASSERT(this->state_ == INITIALIZED || this->state_ == DEFERRED);

  this->state_ = STARTED;
  this->bytes_transferred_ = 0;
  this->error_ = EINPROGRESS;
  return 0;
}

int
ACE_Asynch_Result_Impl::set_deferred ()
{
  //remove assert after tests
  //ACE_ASSERT(this->state_ == STARTED || this->state_ == INITIALIZED);

  this->state_ = DEFERRED;
  this->bytes_transferred_ = 0;
  this->error_ = 0;
  return 0;
}

int
ACE_Asynch_Result_Impl::set_start_error (u_long errcode)
{
  //remove assert after tests
  //ACE_ASSERT(this->state_ == STARTED || this->state_ == INITIALIZED);

  this->state_ = START_ERROR;
  this->bytes_transferred_ = 0;
  this->error_ = errcode;
  return 0;
}

int
ACE_Asynch_Result_Impl::set_ready (ssize_t nbytes,
                                   u_long errcode,
                                   const void *completion_key)
{
  //remove assert after tests
  //ACE_ASSERT(this->state_ == STARTED || this->state_ == INITIALIZED);

  this->state_ = RESULT_READY;

  // do not pass -1 as transfer count
  if (nbytes  > 0 )
    this->bytes_transferred_ = ACE_static_cast (u_long, nbytes);
  else
    this->bytes_transferred_ = 0;

  this->error_ = errcode;
  this->completion_key_ = completion_key;

  return 0;
}

int
ACE_Asynch_Result_Impl::set_posted ()
{
  //remove assert after tests
  //ACE_ASSERT(this->state_ == RESULT_READY ||
  //           this->state_ == INITIALIZED ||
  //           this->state_ == START_ERROR);

  this->state_ = POSTED;
  return 0;
}

int
ACE_Asynch_Result_Impl::set_completed ()
{
  //ACE_ASSERT(this->state_ == RESULT_READY || this->state_ == POSTED);

  this->state_ = COMPLETED;
  return 0;
}

int 
ACE_Asynch_Result_Impl::reuse ()
{
  if (this->flags_ != INTERNAL_RESULT)
    return -1;

  switch (this->state_)
    {
    case STARTED:
      return -1; // cannot be reused;

    case DEFERRED:
      return 0; // already  wait for start

    default:
      break;
    }

  this->state_ = INITIALIZED;
  return 1;
}

int
ACE_Asynch_Result_Impl::is_read_family (void) const
{
  return this->op_code_ & (READ_STREAM|READ_FILE|READ_DGRAM|ACCEPT);
}

int
ACE_Asynch_Result_Impl::is_write_family (void) const
{
  return this->op_code_ & (WRITE_STREAM|WRITE_FILE|WRITE_DGRAM|CONNECT);
}



// **********************************************************************
ACE_Asynch_Result_Impl::~ACE_Asynch_Result_Impl (void)
{
}

ACE_Asynch_Operation_Impl::~ACE_Asynch_Operation_Impl (void)
{
}

// ********** Read Stream ***********************************************
ACE_Asynch_Read_Stream_Impl::~ACE_Asynch_Read_Stream_Impl (void)
{
}

ACE_Asynch_Read_Stream_Result_Impl::ACE_Asynch_Read_Stream_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = READ_STREAM;
}

ACE_Asynch_Read_Stream_Result_Impl::~ACE_Asynch_Read_Stream_Result_Impl (void)
{
}

// ********** Write Stream ***********************************************
ACE_Asynch_Write_Stream_Impl::~ACE_Asynch_Write_Stream_Impl (void)
{
}

ACE_Asynch_Write_Stream_Result_Impl::ACE_Asynch_Write_Stream_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = WRITE_STREAM;
}

ACE_Asynch_Write_Stream_Result_Impl::~ACE_Asynch_Write_Stream_Result_Impl (void)
{
}

// ********** Read File ***********************************************
ACE_Asynch_Read_File_Impl::~ACE_Asynch_Read_File_Impl (void)
{
}

ACE_Asynch_Read_File_Result_Impl::ACE_Asynch_Read_File_Result_Impl (void)
  : ACE_Asynch_Result_Impl (),
    ACE_Asynch_Read_Stream_Result_Impl ()
{
  this->op_code_ = READ_FILE;
}

ACE_Asynch_Read_File_Result_Impl::~ACE_Asynch_Read_File_Result_Impl (void)
{
}

// ********** Write File ***********************************************
ACE_Asynch_Write_File_Impl::~ACE_Asynch_Write_File_Impl (void)
{
}

ACE_Asynch_Write_File_Result_Impl::ACE_Asynch_Write_File_Result_Impl (void)
  : ACE_Asynch_Result_Impl (),
    ACE_Asynch_Write_Stream_Result_Impl ()
{
  this->op_code_ = WRITE_FILE;
}

ACE_Asynch_Write_File_Result_Impl::~ACE_Asynch_Write_File_Result_Impl (void)
{
}

// ********** Accept ***********************************************
ACE_Asynch_Accept_Impl::~ACE_Asynch_Accept_Impl (void)
{
}

ACE_Asynch_Accept_Result_Impl::ACE_Asynch_Accept_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = ACCEPT;
}

ACE_Asynch_Accept_Result_Impl::~ACE_Asynch_Accept_Result_Impl (void)
{
}

// ********** Connect ***********************************************
ACE_Asynch_Connect_Impl::~ACE_Asynch_Connect_Impl (void)
{
}

ACE_Asynch_Connect_Result_Impl::ACE_Asynch_Connect_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = CONNECT;
}


ACE_Asynch_Connect_Result_Impl::~ACE_Asynch_Connect_Result_Impl (void)
{
}

// ********** Transmit File ***********************************************
ACE_Asynch_Transmit_File_Impl::~ACE_Asynch_Transmit_File_Impl (void)
{
}


ACE_Asynch_Transmit_File_Result_Impl::ACE_Asynch_Transmit_File_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = TRANSMIT_FILE;
}

ACE_Asynch_Transmit_File_Result_Impl::~ACE_Asynch_Transmit_File_Result_Impl (void)
{
}

// ********** Read Dgram ***********************************************
ACE_Asynch_Read_Dgram_Impl::~ACE_Asynch_Read_Dgram_Impl (void)
{
}

ACE_Asynch_Read_Dgram_Result_Impl::ACE_Asynch_Read_Dgram_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = READ_DGRAM;
}

ACE_Asynch_Read_Dgram_Result_Impl::~ACE_Asynch_Read_Dgram_Result_Impl (void)
{
}


// ********** Write Dgram ***********************************************
ACE_Asynch_Write_Dgram_Impl::~ACE_Asynch_Write_Dgram_Impl (void)
{
}

ACE_Asynch_Write_Dgram_Result_Impl::ACE_Asynch_Write_Dgram_Result_Impl (void)
  : ACE_Asynch_Result_Impl ()
{
  this->op_code_ = WRITE_DGRAM;
}

ACE_Asynch_Write_Dgram_Result_Impl::~ACE_Asynch_Write_Dgram_Result_Impl (void)
{
}
// *********************************************************************

#endif /* ACE_WIN32 || ACE_HAS_WINCE */
