/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

// POSIX_AIO_Allocator.cpp
// POSIX_AIO_Allocator.cpp,v 4.90 2002/12/28 03:06:31 nanbor Exp

#ifndef ACE_POSIX_AIO_ALLOCATOR_T_C
#define ACE_POSIX_AIO_ALLOCATOR_T_C

#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#include "ace/POSIX_AIO_Allocator.h"



ACE_RCSID(ace, POSIX_AIO_Allocator, "POSIX_AIO_Allocator.cpp,v 4.90 2002/12/28 03:06:31 nanbor Exp")

template <class T, class ACE_LOCK>
ACE_Cached_List_Allocator<T, ACE_LOCK>::ACE_Cached_List_Allocator (void)
  : free_list_ (ACE_PURE_FREE_LIST),
    chunk_size_ (sizeof(T))
{
  chunk_size_ = ACE_MALLOC_ROUNDUP (chunk_size_, ACE_MALLOC_ALIGN);
  
  ACE_DEBUG((LM_DEBUG,
             ACE_LIB_TEXT("ALLOCATOR Chunk size=%u\n"),
             this->chunk_size_           
            )); 
             
}

template <class T, class ACE_LOCK>
ACE_Cached_List_Allocator<T, ACE_LOCK>::~ACE_Cached_List_Allocator (void)
{
  for (;;)
    {
      ACE_Cached_Mem_Pool_Node<T> * ptr = this->free_list_.remove ();

      if (ptr == 0)
        break;

      ACE_OS::free(ptr);
    }
}

template <class T, class ACE_LOCK> ACE_INLINE void *
ACE_Cached_List_Allocator<T, ACE_LOCK>::malloc (size_t nbytes)
{
  // Check if size requested fits within pre-determined size.
  if (nbytes > this->chunk_size_)
    return 0;

  // addr() call is really not absolutely necessary because of the way
  // ACE_Cached_Mem_Pool_Node's internal structure arranged.

  ACE_Cached_Mem_Pool_Node<T> * ptr = this->free_list_.remove ();

  void * p ;

  if (ptr != 0)
	p = ACE_static_cast (void *, ptr);
  else
	p = ACE_OS::malloc (this->chunk_size_);

  //ACE_DEBUG((LM_DEBUG,"+++Malloc size=%d ret=%@ free_chunk=%@\n", 
  //          (int) nbytes, p, ptr));

  return p;		
}

template <class T, class ACE_LOCK> ACE_INLINE void *
ACE_Cached_List_Allocator<T, ACE_LOCK>::calloc (size_t nbytes,
                                           char initial_value)
{
  void * ptr = this->malloc (nbytes);

  if (ptr != 0)
    ACE_OS::memset (ptr, initial_value, this->chunk_size_);

  return ptr;
}

template <class T, class ACE_LOCK> ACE_INLINE void *
ACE_Cached_List_Allocator<T, ACE_LOCK>::calloc (size_t,
                                           size_t,
                                           char)
{
  ACE_NOTSUP_RETURN (0);
}

template <class T, class ACE_LOCK> ACE_INLINE void
ACE_Cached_List_Allocator<T, ACE_LOCK>::free (void * ptr)
{
  //ACE_DEBUG((LM_DEBUG,"---Free ptr=%@\n",  ptr));

  if (ptr != 0)
    this->free_list_.add (ACE_static_cast(ACE_Cached_Mem_Pool_Node<T> *, ptr));
}

#endif /* ACE_POSIX_AIO_ALLOCATOR_T_C */
