/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
// $Id:  $

#include "ace/POSIX_AIO_Config.h"

#if defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)

#include "ace/ACE.h"
#include "ace/Log_Msg.h"

#include "ace/POSIX_AIO_Dispatcher.h"
#include "ace/POSIX_AIO_Dedicated_Processor.h"

#include "ace/POSIX_AIO_STD_Provider.h"
#include "ace/POSIX_AIO_Select_Provider.h"
#include "ace/POSIX_AIO_SUN_Provider.h"
#include "ace/POSIX_AIO_LINUX_Provider.h"

ACE_POSIX_AIO_Config::ACE_POSIX_AIO_Config (size_t  max_num_aio,
                                            int     processor_type, 
                                            u_long  processor_flags, 
                                            int     provider_type,   
                                            u_long  provider_flags,  
                                            int     ws_type,         
                                            int     wait_sig_num,    
                                            int     is_type,         
                                            int     interrupt_sig_num)
  : max_num_aio_       (max_num_aio),
    processor_type_    (processor_type), 
    processor_flags_   (processor_flags),
    provider_type_     (provider_type),   
    provider_flags_    (provider_flags),
    ws_type_           (ws_type),   
    wait_sig_num_      (wait_sig_num), 
    is_type_           (is_type),
    interrupt_sig_num_ (interrupt_sig_num)
{
}

ACE_POSIX_AIO_Config::~ACE_POSIX_AIO_Config (void)
{
}

void
ACE_POSIX_AIO_Config::dump (void) const
{
  ACE_TRACE ("ACE_Addr::dump");

  const ACE_TCHAR * str_tmp1 ;
  const ACE_TCHAR * str_tmp2 ;

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("Max. number aios : %u\n"),
              (unsigned int)this->max_num_aio_));

  switch (this->processor_type_)
    {
    case PCT_NONE:      str_tmp1 = ACE_LIB_TEXT("DUMMY (not exist)"); break;
    case PCT_DEDICATED: str_tmp1 = ACE_LIB_TEXT("DEDICATED"); break;
    case PCT_SHARED:    str_tmp1 = ACE_LIB_TEXT("SHARED"); break;
    default:            str_tmp1 = ACE_LIB_TEXT("??????"); break;
    }
  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("AIO Processor type  : %d %s\n"), 
              this->processor_type_,
              str_tmp1));

  switch (this->provider_type_)
    {
    case PVT_NONE:      str_tmp1 = ACE_LIB_TEXT("DUMMY (not exist)"); break;
    case PVT_POSIX:     str_tmp1 = ACE_LIB_TEXT("POSIX"); break;
    case PVT_SUN:       str_tmp1 = ACE_LIB_TEXT("SUN"); break;
    case PVT_SELECT:    str_tmp1 = ACE_LIB_TEXT("SELECT"); break;
    case PVT_LINUX:     str_tmp1 = ACE_LIB_TEXT("LINUX"); break;
    default:            str_tmp1 = ACE_LIB_TEXT("??????"); break;
    }
  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("AIO Provider type   : %d %s\n"), 
              this->provider_type_,
              str_tmp1));

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("AIO Provider Flags  : %X\n"),
              (unsigned int) this->provider_flags_));

  str_tmp1 = (this->provider_flags_ & PVF_AIO_START_ANY_THREAD) 
           ? ACE_LIB_TEXT("ANY THREAD")
           : ACE_LIB_TEXT("ONLY LEADER");
              
  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("      start aio in  : %s\n"), 
              str_tmp1));

  str_tmp2 = (this->provider_flags_ & PVF_AIO_START_INTERRUPT_WAIT) 
           ? ACE_LIB_TEXT("YES")
           : ACE_LIB_TEXT("NO");

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("  interrupt leader  : %s\n"),
              str_tmp2));       

  switch (this->ws_type_)
    {
    case WST_DEFAULT:   str_tmp1 = ACE_LIB_TEXT("DEFAULT"); break;
    case WST_NATIVE:    
      if (this->provider_type_ == PVT_SELECT)
        str_tmp1 = ACE_LIB_TEXT("SELECT");
      else
        str_tmp1 = ACE_LIB_TEXT("NATIVE");
      break;
    case WST_SIGWAIT:   
      if (this->provider_type_ == PVT_SELECT)
        str_tmp1 = ACE_LIB_TEXT("LINUXRT");
      else
        str_tmp1 = ACE_LIB_TEXT("SIGWAIT");
      break;
    case WST_SGI:       str_tmp1 = ACE_LIB_TEXT("SGI");     break;
    case WST_DEVPOLL:   str_tmp1 = ACE_LIB_TEXT("DEVPOLL/EPOLL"); break;
    case WST_POLL:      str_tmp1 = ACE_LIB_TEXT("POLL");    break;
    case WST_OTHER:     str_tmp1 = ACE_LIB_TEXT("OTHER");   break;
    default:            str_tmp1 = ACE_LIB_TEXT("??????");  break;
    }

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("Wait Strategy       : %d %s\n"), 
              this->ws_type_,
              str_tmp1));

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("Wait Parameter      : %d\n"),
              this->wait_sig_num_));

  switch (this->is_type_)
    {
    case IST_DEFAULT:   str_tmp1 = ACE_LIB_TEXT("DEFAULT"); break;
    case IST_PIPE:      str_tmp1 = ACE_LIB_TEXT("PIPE");    break;
    case IST_SIGNAL:    str_tmp1 = ACE_LIB_TEXT("SIGNAL");  break;
    case IST_SGI:       str_tmp1 = ACE_LIB_TEXT("SGI");     break;
    case IST_OTHER:     str_tmp1 = ACE_LIB_TEXT("OTHER");   break;
    default:            str_tmp1 = ACE_LIB_TEXT("??????");  break;
    }

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("Interrupt Strategy  : %d %s\n"), 
              this->is_type_,
              str_tmp1));

  ACE_DEBUG ((LM_DEBUG, 
              ACE_LIB_TEXT ("Interrupt Parameter : %d\n"),
              this->interrupt_sig_num_));

}

int
ACE_POSIX_AIO_Config::processor_type (void) const 
{ 
  return this->processor_type_;
} 

u_long
ACE_POSIX_AIO_Config::processor_flags (void) const 
{
  return this->processor_flags_;
}

int
ACE_POSIX_AIO_Config::provider_type (void) const 
{
  return this->provider_type_;
}

u_long
ACE_POSIX_AIO_Config::provider_flags (void) const 
{
  return this->provider_flags_;
}

int
ACE_POSIX_AIO_Config::ws_type (void) const 
{
  return this->ws_type_;
}

int
ACE_POSIX_AIO_Config::wait_sig_num (void) const 
{ 
  return this->wait_sig_num_;
}

int
ACE_POSIX_AIO_Config::is_type (void) const
{
  return this->is_type_;
}

int
ACE_POSIX_AIO_Config::interrupt_sig_num (void) const 
{
  return this->interrupt_sig_num_;
}

size_t
ACE_POSIX_AIO_Config::max_num_aio (void) const 
{ 
  return this->max_num_aio_;
}

ACE_POSIX_AIO_Provider *
ACE_POSIX_AIO_Config::create_provider (void) const
{
  ACE_POSIX_AIO_Provider * provider = 0;
  switch (this->provider_type_)
    {
    case PVT_POSIX:
      ACE_NEW_RETURN (provider,
                      ACE_POSIX_AIO_STD_Provider (*this),
                      0);
      break;

    case PVT_SUN:
#if defined(sun)
      ACE_NEW_RETURN (provider,
                      ACE_POSIX_AIO_SUN_Provider (*this),
                      0);
#endif /* sun */
      break;
      
    case PVT_LINUX:
#if defined(linux) && defined(ACE_HAS_LINUX_LIBAIO)
      ACE_NEW_RETURN (provider,
                      ACE_POSIX_AIO_LINUX_Provider (*this),
                      0);
#endif /* linux && ACE_HAS_LINUX_LIBAIO */
      break;

    case PVT_SELECT:
      ACE_NEW_RETURN (provider,
                      ACE_POSIX_AIO_Select_Provider (*this),
                      0);
      break;

    default:
      break;
    }

  return provider;
}

ACE_POSIX_AIO_Processor *
ACE_POSIX_AIO_Config::create_processor 
                       (ACE_POSIX_AIO_Dispatcher & dispatcher) const
{
  ACE_POSIX_AIO_Processor * processor = 0;

  switch (this->processor_type_)
    {
    case PCT_DEDICATED:
      ACE_NEW_RETURN (processor,
                      ACE_POSIX_AIO_Dedicated_Processor(dispatcher,*this),
                      0);
      break;

    case PCT_SHARED:
      ACE_NEW_RETURN (processor,
                      ACE_POSIX_AIO_Processor(dispatcher,*this),
                      0);
      break;

    default:
      break;
    }

  return processor;
}


ACE_POSIX_AIO_Interrupt_Strategy *
ACE_POSIX_AIO_Config::create_interrupt_strategy 
                       (ACE_POSIX_AIO_Provider & provider) const
{
  /// This method provides creation only
  /// of provider-independed Interrupt_Strategies 
  /// such as Pipe/Signal Interrupt_Startegies.
  
  ACE_POSIX_AIO_Interrupt_Strategy * new_strategy = 0;
  switch (this->is_type_)
    {
    case IST_SIGNAL:
      ACE_NEW_RETURN (new_strategy,
                      ACE_POSIX_AIO_Signal_Interrupt_Strategy(provider),
                      0);
      break;

    case IST_DEFAULT:
    case IST_PIPE:
    default:
      ACE_NEW_RETURN (new_strategy,
                      ACE_POSIX_AIO_Pipe_Interrupt_Strategy(provider),
                      0);
      break;
    }

  return new_strategy;
}

  

#endif /* ACE_HAS_AIO_CALLS */
