/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
//=============================================================================
/**
 *
 *  @file    POSIX_AIO_Config.h
 *
 *  $Id: $
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
//=============================================================================

#ifndef ACE_POSIX_AIO_CONFIG_H
#define ACE_POSIX_AIO_CONFIG_H

#include "ace/pre.h"
#include "ace/config-all.h"


#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#include "ace/OS.h"

#if defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)

#define ACE_AIO_MAX_SIZE     2048
#define ACE_AIO_DEFAULT_SIZE 1024
#define ACE_AIO_NO_LIMIT     (ACE_static_cast(size_t,-1))


//Forward declarations

class ACE_POSIX_AIO_Dispatcher;
class ACE_POSIX_AIO_Processor;
class ACE_POSIX_AIO_Provider;
class ACE_POSIX_AIO_Wait_Strategy;
class ACE_POSIX_AIO_Interrupt_Strategy;


/**
 * @class ACE_POSIX_AIO_Config
 *
 * @brief POSIX implementation .
 */

class ACE_Export ACE_POSIX_AIO_Config 
{

public:

  // AIO_Processor Types
  enum Processor_Type  
    {
      PCT_NONE      = 0,  // Processor not exist
      PCT_SHARED    = 1,  // handle events in 
                          // user thread 
      PCT_DEDICATED = 2   // separate permanent thread
                          // for handle events
    };

  enum Processor_Flags   // for the future extentions
    {
      PCF_ZERO = 0x0000 
    };



  // AIO_Provider Types
  enum Provider_Type           
    {
      PVT_NONE    = 0,  // Provider not exist
      PVT_POSIX   = 1,  // POSIX Standart AIO api
      PVT_SUN     = 2,  // SUN native AIO api
      PVT_SELECT  = 3,  // Emulation AIO via select, poll, /dev/poll, epoll ...
      PVT_LINUX   = 4   // Linux libaio apo io_XXXXX
    };

  enum Provider_Flags
    {
      PVF_AIO_START_ANY_THREAD     = 0x0001,  // AIO can start in any thread, 
                                              // othewise on in leader thread

      PVF_AIO_START_INTERRUPT_WAIT = 0x0002   // Start AIO should interrupt waiter/leader 
                                              // to update wait state. For example,
                                              // re-issue aio_suspend() with new params.
    };


 
  //  Wait Strategy Types
  enum Wait_Strategy_Type  
    {
      WST_DEFAULT = 0,

      WST_NATIVE  = 1,              // native Wait_Stategy, i.e provider-specific
      
      // Provider PVT_POSIX 
      //   (POSIX_AIO)              //     Wait_Strategy based on
      //                            // --------------------------------------------
      WST_AIO_SUSPEND = WST_NATIVE, // aio_suspend() (old POSIX_AIOCB_Proactor)
      WST_SIGWAIT     = 2,          // sigwaitinfo() (old POSIX_SIG_Proactor)
      WST_SGI         = 3,          // wait notification from callback, i.e.
                                    // wait on semaphore (old POSIX_CB_Proactor)
                                    
      // Provider PVT_SUN
      //   (SUN AIO)
      //                            // --------------------------------------------
      WST_AIOWAIT     = WST_NATIVE, // aiowait()  (old SUN_Proactor)
      
      // Provider PVT_SELECT
      //   (Emulation AIO)          //    (new Select Proactor)
      //                            // --------------------------------------------
      WST_SELECT     = WST_NATIVE,  // select()      all POSIX systems
      WST_POLL       = 5,           // poll ()       all POSIX systems
      WST_LINUXRT    = WST_SIGWAIT, // sigwaitinfo() Linux RT signals
      WST_EPOLL      = 4,           // /sys/epoll    Linux kernel 2.6
      WST_DEVPOLL    = WST_EPOLL,   // /dev/poll     SUN 5.8 and higher
      WST_KQUEUE     = 6,           // kqueue ()     FreeBSD
      
      // Provider PVT_LINUX
      //   (Linux libaio.h)         //    (new Linux Proactor)
      //                            // --------------------------------------------
      WST_LINUX_IO    = WST_NATIVE, // io_getevents()  Linux kernel 2.6

      WST_OTHER   = -1              // Unknown
    };


  // Interrupt Strategy Types
  enum Interrupt_Strategy_Type  
    {
      IST_DEFAULT = 0,
      IST_PIPE    = 1,  // notification pipe strategy 
      IST_SIGNAL  = 2,  // signal to the leader thread (thr_kill)
      IST_SGI     = 3,  // SGI - semaphore release in AIO Callback 
      IST_OTHER   = -1  // Unknown
    };

  ACE_POSIX_AIO_Config (size_t  max_num_aio     = ACE_AIO_NO_LIMIT,
                        int     processor_type  = PCT_SHARED,
                        u_long  processor_flags = PCF_ZERO,
                        int     provider_type   = PVT_POSIX,
                        u_long  provider_flags  = PVF_AIO_START_ANY_THREAD 
                                                | PVF_AIO_START_INTERRUPT_WAIT,
                        int     ws_type         = WST_DEFAULT,
                        int     wait_sig_num    = ACE_SIGRTMIN,
                        int     is_type         = IST_DEFAULT,
                        int     interrupt_sig_num = SIGUSR1);

  ~ACE_POSIX_AIO_Config (void);

  
  
  /// factory method - create interrupt strategy
  /// Parameter is_type - desired interrupt strategy type.
  /// This method provides creation only
  /// of provider-independed Interrupt_Strategies 
  /// such as Pipe/Signal Interrupt_Startegies.

  ACE_POSIX_AIO_Interrupt_Strategy *
    create_interrupt_strategy (ACE_POSIX_AIO_Provider & provider) const;

  ACE_POSIX_AIO_Provider *
    create_provider (void) const;

  ACE_POSIX_AIO_Processor *
    create_processor (ACE_POSIX_AIO_Dispatcher & dispatcher) const;

  void    dump (void) const;

  size_t  max_num_aio (void) const;

  int     processor_type (void) const;
  u_long  processor_flags (void) const;
  
  int     provider_type (void) const;
  u_long  provider_flags (void) const;

  int     ws_type (void) const;
  int     wait_sig_num (void) const;

  int     is_type (void) const;
  int     interrupt_sig_num (void) const;


protected:

  size_t  max_num_aio_;

  int     processor_type_;
  u_long  processor_flags_;
  
  int     provider_type_;
  u_long  provider_flags_;

  int     ws_type_; 
  int     wait_sig_num_;     

  int     is_type_;
  int     interrupt_sig_num_;   // used for IST_SIGNAL

};

#endif /* ACE_HAS_AIO_CALLS || ACE_HAS_AIO_EMULATION */

#include "ace/post.h"

#endif /* ACE_POSIX_AIO_CONFIG_H */
