/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
// $Id:  $

#include "ace/POSIX_AIO_Dev_Poll_Strategy.h"

#if defined (ACE_HAS_AIO_CALLS) || defined (ACE_HAS_AIO_EMULATION)

#if defined (ACE_HAS_DEV_POLL)  

//#if defined (sun)

#include /**/ <sys/devpoll.h>

//#endif /*sun*/


#include "ace/Log_Msg.h"
#include "ace/Addr.h"
#include "ace/ACE.h"

#define POLL_BUF_SIZE 100

/**
 *****************************************************************
 *  POSIX Dev_Poll based AIO Wait Strategy
 *
 *****************************************************************
 */
ACE_POSIX_AIO_Dev_Poll_Strategy::~ACE_POSIX_AIO_Dev_Poll_Strategy()
{
  this->fini ();
}

ACE_POSIX_AIO_Dev_Poll_Strategy::ACE_POSIX_AIO_Dev_Poll_Strategy 
  (ACE_POSIX_AIO_Select_Provider & provider)
  : ACE_POSIX_AIO_Select_Strategy_Base (provider),
    poll_fd_ (ACE_INVALID_HANDLE),
    dp_fds_ (0),
    start_pfds_ (0),
    num_events_ (0)
{
}

int 
ACE_POSIX_AIO_Dev_Poll_Strategy::init (void)
{
  if (this->alloc_repository (ACE_AIO_MAX_SIZE, 0) != 0)
    return -1;;

  ACE_DEBUG ((LM_DEBUG,
    ACE_LIB_TEXT("(%t)ACE_POSIX_AIO_Dev_Poll_Strategy size=%d\n"), 
    (int) this->size_));


  // Allocate the array before opening the device to avoid a potential
  // resource leak if allocation fails.
  ACE_NEW_RETURN (this->dp_fds_,
                  pollfd[POLL_BUF_SIZE+1],
                  -1);

  // Open the `/dev/poll' character device.
  this->poll_fd_ = ACE_OS::open ("/dev/poll", O_RDWR);
  if (this->poll_fd_ == ACE_INVALID_HANDLE)
    return -1;
  

  return 0;
}  

int 
ACE_POSIX_AIO_Dev_Poll_Strategy::fini (void)
{
  delete [] this->dp_fds_;
  this->dp_fds_ = 0;

  if (this->poll_fd_ != ACE_INVALID_HANDLE)
    {
      ACE_OS::close (this->poll_fd_);
    }

  this->poll_fd_ = ACE_INVALID_HANDLE;
  this->start_pfds_ = 0;
  this->num_events_ = 0;
  
  this->free_repository ();

  return 0;
}

int 
ACE_POSIX_AIO_Dev_Poll_Strategy::on_aio_start (ACE_POSIX_Asynch_Result * result)
{
  ACE_HANDLE handle = result->aio_fildes;

  int blk_mode = ACE_BIT_DISABLED (ACE::get_flags (handle), ACE_NONBLOCK);

  // Set the handle into non-blocking mode if it's not already in it.
  if (blk_mode)
    ACE::set_flags (handle, ACE_NONBLOCK);

  if (result->op_code () == ACE_Asynch_Result_Impl::CONNECT)
    {
      if (this->pending_writes_[handle] == IO_READY)
          this->pending_writes_[handle] = 0;

      if (this->pending_reads_ [handle] == IO_READY)
          this->pending_reads_ [handle] = 0;
    }

  return 0; 
}


int
ACE_POSIX_AIO_Dev_Poll_Strategy::prepare_to_wait (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_TRACE ("ACE_POSIX_AIO_Dev_Poll_Strategy::prepare_to_wait");

  ACE_UNUSED_ARG (started_list);

  if (this->num_events_ > 0)
    return WAIT_COMPLETED; 

  return WAIT_ENABLED;
}

int
ACE_POSIX_AIO_Dev_Poll_Strategy::wait (ACE_Time_Value * wait_time)
{
  ACE_TRACE ("ACE_POSIX_AIO_Dev_Poll_Strategy::wait");

  long timeout = -1L;
  
  if (wait_time != 0 )
  {
  	 timeout =  ACE_static_cast (long, wait_time->msec ());
  }


  struct dvpoll dvp;

  dvp.dp_fds = this->dp_fds_;
  dvp.dp_nfds = POLL_BUF_SIZE;
  dvp.dp_timeout = timeout;  // Milliseconds

  this->fd_info_.set(ACE_INVALID_HANDLE, 0);
  
  // Poll for events
  this->num_events_ = ACE_OS::ioctl (this->poll_fd_, DP_POLL, &dvp);

  // Retrieve the results from the pollfd array.
  this->start_pfds_ = dvp.dp_fds;


  if (this->num_events_ > 0)
    {
      this->fd_info_.set (this->start_pfds_->fd,
                           this->start_pfds_->revents);
      
      return WAIT_COMPLETED;
    }

  if (this->num_events_ == 0)
    return WAIT_TIMEOUT;

  this->num_events_ = 0;
 
  switch (errno)
    {
    case EINTR :    // Interrupted call
      return WAIT_INTERRUPTED;

    default:
      break;
    }
  return WAIT_ERROR;
}

ACE_POSIX_Asynch_Result *
ACE_POSIX_AIO_Dev_Poll_Strategy::get_next_completed_result 
  (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_UNUSED_ARG (started_list);

  for (;;)
    {

      ACE_HANDLE handle = this->fd_info_.handle();

      if (handle == ACE_INVALID_HANDLE)
        return 0;

      int rc = this->check (this->fd_info_);

      if (rc)
        {
          ACE_POSIX_Asynch_Result * result = this->fd_info_.get_result();
          ACE_ASSERT (result !=0);
          return result;
        }

      if ((this->fd_info_.events_to_clear () != 0 ) ||
          (this->fd_info_.events_to_set () != 0 ))
        {
          this->modify_poll (handle);
        }

      if ( --this->num_events_ <= 0 )
        break;


      ++this->start_pfds_ ;
      
      this->fd_info_.set (this->start_pfds_->fd,
                           this->start_pfds_->revents);

    }

  return 0;
}

int 
ACE_POSIX_AIO_Dev_Poll_Strategy::modify_poll (ACE_HANDLE handle)
{
  struct pollfd pfd[2];
  int len = sizeof (pfd[0]);

  short events  = 0;

  if (this->pending_writes_[handle] != 0 && 
      this->pending_writes_[handle] != IO_READY )
    events |= POLLOUT;

  if (this->pending_reads_[handle] != 0 && 
      this->pending_reads_[handle] != IO_READY )
    events |= POLLIN;


  // Apparently events cannot be updated on-the-fly on Solaris so
  // remove the existing events, and then add the new ones.

  pfd[0].fd      = handle;
  pfd[0].events  = POLLREMOVE;
  pfd[0].revents = 0;

  if ( events != 0)
    {
      pfd[1].fd      = handle;
      pfd[1].events  = events;
      pfd[1].revents = 0;
      len = sizeof(pfd);
    }

  // Change the events associated with the given file descriptor.
  if (ACE_OS::write (this->poll_fd_,
                     pfd,
                     len) != len )
    ACE_ERROR_RETURN ((LM_ERROR,
      ACE_LIB_TEXT("ACE_POSIX_AIO_Dev_Poll_Strategy::modify poll ")
      ACE_LIB_TEXT("write to /dev/poll failed")),
      -1);

  return 0;
}
    
// *****************************************************************
//  Declare interest
//  Return code :
//    1 - we should interrupt leader to update wait state
//    0 - we do not have to interrupt leader
// *****************************************************************

int
ACE_POSIX_AIO_Dev_Poll_Strategy::declare_interest (ACE_HANDLE handle, 
                                                   u_int mask)
{  
  ACE_UNUSED_ARG (mask);
  this->modify_poll (handle);

  return 0;
}

int
ACE_POSIX_AIO_Dev_Poll_Strategy::clear_interest (ACE_HANDLE handle, 
                                                 u_int mask)
{  
  ACE_UNUSED_ARG (mask);
  this->modify_poll (handle);
  return 0;
}


#endif /* ACE_HAS_DEV_POLL */
#endif /* ACE_HAS_AIO_CALLS  || ACE_HAS_AIO_EMULATION */
