/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
// $Id: $

#include "ace/POSIX_AIO_Dispatcher.h"

#if defined (ACE_HAS_AIO_CALLS)  || defined(ACE_HAS_AIO_EMULATION)

#include "ace/Log_Msg.h"
#include "ace/Auto_IncDec_T.h"


#include "ace/POSIX_AIO_Processor.h"

// ********************************************************************

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)

template class ACE_Reverse_Lock<ACE_SYNCH_MUTEX>;
template class ACE_Guard<ACE_Reverse_Lock<ACE_SYNCH_MUTEX> >;
template class ACE_Double_Linked_List<Thread_Dispatcher>;
template class ACE_Double_Linked_List_Iterator_Base<Thread_Dispatcher>;
template class ACE_Double_Linked_List_Iterator<Thread_Dispatcher>;

#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)

#pragma instantiate ACE_Reverse_Lock<ACE_SYNCH_MUTEX>
#pragma instantiate ACE_Guard<ACE_Reverse_Lock<ACE_SYNCH_MUTEX> >
#pragma instantiate ACE_Double_Linked_List<Thread_Dispatcher>
#pragma instantiate ACE_Double_Linked_List_Iterator_Base<Thread_Dispatcher>
#pragma instantiate ACE_Double_Linked_List_Iterator<Thread_Dispatcher>

#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */

typedef ACE_Reverse_Lock<ACE_SYNCH_MUTEX> REVERSE_MUTEX;

// ********************************************************************

ACE_SYNCH_MUTEX Thread_Dispatcher::dummy_mutex_;

Thread_Dispatcher::Thread_Dispatcher (ACE_POSIX_AIO_Dispatcher * dispatcher)
  : dispatcher_ (dispatcher),
    owner_(0),
    condition_ (dispatcher != 0 ? dispatcher->mutex() : dummy_mutex_),
    disp_result_ (0),
    processor_(0),
    next_(0),
    prev_(0),
    num_events_(0)
{
  ;
}

Thread_Dispatcher::~Thread_Dispatcher ()
{
  this->unlink();

  ACE_ASSERT ( this->list_waiters_.size() == 0);
}

void
Thread_Dispatcher::unlink()
{ 
  if (this->owner_ != 0)
    this->owner_->remove (this);

  this->owner_ = 0;
}

void
Thread_Dispatcher::link (Thread_Dispatcher_List & list)
{ 
  ACE_ASSERT(this->owner_ == 0);
  this->owner_ = &list;
  this->owner_->insert_head(this);
}

void
Thread_Dispatcher::push_result(ACE_POSIX_Asynch_Result * result)
{
  if (this->disp_result_ == 0)
    {
      this->disp_result_ = result;
      return;
    }

  ACE_ASSERT (this->disp_result_->aio_fildes == result->aio_fildes);

  //this->list_waiters_.insert_tail (result);
  this->list_waiters_.add_result (result);

  return;
}

ACE_POSIX_Asynch_Result * 
Thread_Dispatcher::pop_result(void)
{
  //return this->list_waiters_.delete_head ();

  ACE_POSIX_Asynch_Result* result = this->list_waiters_.get_first();

  if (result != 0)
    this->list_waiters_.remove_result (result);

  this->disp_result_ = result;
  return result;
}


// ********************************************************************

ACE_POSIX_AIO_Dispatcher::ACE_POSIX_AIO_Dispatcher (ACE_SYNCH_MUTEX & mutex)
  : mutex_                (mutex),
    thr_mngr_             (),
//    condition_            (mutex),
//    condition_            (0),   // if semaphore
    num_active_threads_   (0),
    prime_processor_      (0),
    second_processor_     (0)
{
  ;
}

// Destructor.
ACE_POSIX_AIO_Dispatcher::~ACE_POSIX_AIO_Dispatcher (void)
{
  this->clear_result_queue();

  for (;;)
    {
      Thread_Dispatcher * thr_dispatcher = this->free_list_.delete_head ();

      if (thr_dispatcher == 0)
        break;

      delete thr_dispatcher;
    }

  ACE_ASSERT (this->leaders_.size() == 0);
  ACE_ASSERT (this->dispatchers_.size() == 0);
  ACE_ASSERT (this->followers_.size() == 0);
}

ACE_SYNCH_MUTEX &
ACE_POSIX_AIO_Dispatcher::mutex ()
{
  return this->mutex_;
}

ACE_Thread_Manager &
ACE_POSIX_AIO_Dispatcher::thread_manager()
{
  return this->thr_mngr_;
}

int 
ACE_POSIX_AIO_Dispatcher::init (ACE_POSIX_AIO_Processor * prime_processor,
                                ACE_POSIX_AIO_Processor * second_processor)

{
  if (this->prime_processor_ != 0 || this->second_processor_ != 0)
    return -1;

  this->prime_processor_ = prime_processor;
  this->second_processor_ = second_processor;

  ACE_DEBUG ((LM_DEBUG,
              ACE_LIB_TEXT("POSIX_AIO_Dispatcher::init: prime=%@ second=%@\n"),
              this->prime_processor_,
              this->second_processor_));

  return 0;
}

int 
ACE_POSIX_AIO_Dispatcher::fini ()
{

  this->prime_processor_ = 0;
  this->second_processor_ = 0;

  this->clear_result_queue ();

  return 0;
}

void 
ACE_POSIX_AIO_Dispatcher::dump (const ACE_TCHAR * txt)
{
  if (txt == 0)
    txt = ACE_LIB_TEXT("Dump");

  
  ACE_DEBUG((LM_DEBUG,
    ACE_LIB_TEXT("(%t) %s A=%u L=%u F=%u D=%u Q=%u\n"),
    txt,
    (u_int) this->num_active_threads_,
    (u_int) this->leaders_.size(),
    (u_int) this->followers_.size(),
    (u_int) this->dispatchers_.size(),
    (u_int) this->completed_list_.size()));
}



void 
ACE_POSIX_AIO_Dispatcher::get_loading (size_t & num_followers,
                                       size_t & num_dispatchers,
                                       size_t & queue_size) const
{
  num_followers = this->followers_.size();
  num_dispatchers = this->dispatchers_.size();
  queue_size = this->completed_list_.size();
}


int
ACE_POSIX_AIO_Dispatcher::post_result (ACE_POSIX_Asynch_Result *result,
                                       int notify_flg)
{
  result->set_posted ();

  //this->completed_list_.insert_tail (result);

  this->completed_list_.add_result (result);

  return this->notify (notify_flg);
}

ACE_POSIX_Asynch_Result * 
ACE_POSIX_AIO_Dispatcher::getq_result (void)
{
  //ACE_POSIX_Asynch_Result* result = this->completed_list_.delete_head();

  ACE_POSIX_Asynch_Result* result = this->completed_list_.get_first();

  if (result != 0)
    this->completed_list_.remove_result (result);

  return result;
}

int 
ACE_POSIX_AIO_Dispatcher::clear_result_queue (void)
{
  int ret_val = 0;
  ACE_POSIX_Asynch_Result* result = 0;

  while ((result = this->getq_result ()) != 0)
    {
      ACE_POSIX_Asynch_Result::free (result);
      ++ret_val;
    }

  return ret_val;
}


Thread_Dispatcher * 
ACE_POSIX_AIO_Dispatcher::find_dispatcher(ACE_HANDLE handle)
{
  if (handle == ACE_INVALID_HANDLE)
    return 0;

  Thread_Dispatcher_Iterator itr(this->dispatchers_);

  Thread_Dispatcher * info;

  for (itr.first(); 
       (info = itr.next()) != 0;
       itr.advance())
    {
       if (info->disp_result_->aio_fildes == handle)
         return info;
    }

  return 0;
}

int 
ACE_POSIX_AIO_Dispatcher::is_handle_in_dispatch(ACE_HANDLE handle,
                                                ACE_POSIX_AIO_Processor * processor)
{
   Thread_Dispatcher * ptr_info = this->find_dispatcher (handle);

   if (!ptr_info)
     return 0;
   
   ptr_info->processor_ = processor;
   
   return 1;
}

  

int 
ACE_POSIX_AIO_Dispatcher::process_result_queue (Thread_Dispatcher & thr_dispatcher)
{
  ACE_POSIX_Asynch_Result* result = 0;

  //if (this->completed_list_.size() == 0)
  //  return (thr_dispatcher.num_events_ != 0) ? 1 : 0;

  for (;;)
    {
      thr_dispatcher.processor_ = 0;

      result = this->getq_result();
      if (result == 0)
        break;

      ACE_HANDLE handle = result->aio_fildes;

      Thread_Dispatcher * handle_disp = this->find_dispatcher (handle);

      if (handle_disp != 0)
        {
          handle_disp->push_result (result);
          continue;
        }

      thr_dispatcher.link (this->dispatchers_);
      thr_dispatcher.push_result(result);

      
      for (;;)
        {
          ACE_POSIX_Asynch_Result* result_next = 
            this->completed_list_.get_first ();

          if (result_next == 0)
            break;

          if (result_next->aio_fildes == handle)
            {
              this->completed_list_.remove_result (result_next);
              thr_dispatcher.push_result (result_next);
              continue;
            }

          this->notify_follower();
          break;
        }
         
    
      for ( ; result != 0; result = thr_dispatcher.pop_result())
        {
          int rc = this->dispatch_result (result);

          if (rc !=0 )
            ACE_ERROR((LM_ERROR, 
              ACE_LIB_TEXT("(%t) ACE_POSIX_AIO_Dispatcher::process_result_queue:")
              ACE_LIB_TEXT(" dispatch_result failed\n")));

          ACE_POSIX_Asynch_Result::free (result);

          ++thr_dispatcher.num_events_;
        }

      if (thr_dispatcher.processor_ != 0)
        {
          thr_dispatcher.processor_->
            interrupt_leader (ACE_POSIX_AIO_Processor::IR_NEW_STARTED_AIO);
        }

      thr_dispatcher.unlink ();
    }


  //ACE_DEBUG((LM_DEBUG,"(%t) hq NumEvents=%d\n", 
  //           thr_dispatcher.num_events));

  return (thr_dispatcher.num_events_ != 0) ? 1 : 0;
}

int
ACE_POSIX_AIO_Dispatcher::dispatch_result (ACE_POSIX_Asynch_Result * result)
{
  result->set_completed ();

  REVERSE_MUTEX anti_mutex (this->mutex_);

  ACE_MT (ACE_GUARD_RETURN (REVERSE_MUTEX,
                            ace_mon,
                            anti_mutex, 
                            -1));

  // Call completion hook
  result->complete (result->bytes_transferred (),
                    result->success (),
                    result->completion_key (),
                    result->error ());

  return 0;
}

int
ACE_POSIX_AIO_Dispatcher::notify (int notify_flg)
{
  int rc = 0;

  switch (notify_flg)
    {
    case NOTIFY_STRONG:
      rc = this->notify_follower ();
      if (rc == 0) // no followers
        rc = this->notify_leader ();
      break;

    case NOTIFY_FOLLOWER:
      rc = this->notify_follower ();
      break;

    case NOTIFY_NONE:
    default:
      break;
    }

  return (rc >= 0) ? 0 : -1;
}

int
ACE_POSIX_AIO_Dispatcher::notify_leader (void)
{
  if (this->leaders_.size() == 0)
    return 0;

  Thread_Dispatcher_Iterator itr(this->leaders_);

  itr.first(); 
      
  Thread_Dispatcher * thr_dispatcher = itr.next();

  ACE_ASSERT (thr_dispatcher != 0);
  ACE_ASSERT (thr_dispatcher->processor_ != 0);

  if (thr_dispatcher->processor_->
            interrupt_leader (ACE_POSIX_AIO_Processor::IR_BREAK) !=0)
        ACE_ERROR_RETURN ((LM_ERROR, 
          ACE_LIB_TEXT("(%t) ACE_POSIX_AIO_Dispatcher::notify_leader failed")),
          -1);

  return 1;
}

int
ACE_POSIX_AIO_Dispatcher::notify_follower (void)
{
  // notify only follower speed up processing
  // do not not interrupt the leader/aio_processor.

  if (this->followers_.size() == 0)
    return 0;

  Thread_Dispatcher_Iterator itr(this->followers_);

  itr.first(); 
      
  Thread_Dispatcher * thr_dispatcher = itr.next();

  ACE_ASSERT (thr_dispatcher != 0);

  if (thr_dispatcher->condition_.signal() !=0)
    ACE_ERROR_RETURN ((LM_ERROR, 
      ACE_LIB_TEXT("(%t) ACE_POSIX_AIO_Dispatcher::notify_follower failed")),
     -1);

  return 1;
}

Thread_Dispatcher * 
ACE_POSIX_AIO_Dispatcher::allocate_thr_dispatcher (void)
{
  Thread_Dispatcher * thr_dispatcher = this->free_list_.delete_head();

  if (thr_dispatcher != 0)
    return thr_dispatcher;

  ACE_NEW_RETURN (thr_dispatcher,
                  Thread_Dispatcher(this),
                  0);

  return thr_dispatcher;
}

void
ACE_POSIX_AIO_Dispatcher::free_thr_dispatcher (Thread_Dispatcher * thr_dispatcher)
{
  ACE_ASSERT (thr_dispatcher != 0);

  this->free_list_.insert_head (thr_dispatcher);
}
  

int 
ACE_POSIX_AIO_Dispatcher::handle_aio_events (ACE_Time_Value * wait_time,
                                             ACE_POSIX_AIO_Processor * processor,
                                             Thread_Dispatcher & thr_dispatcher)
{
  // Return codes
  // -1   errors
  //  0   there were no processing 
  //      as aio_processor could not obtain leadership 
  //  1   aio_processor handled some events 
  //      num_events was inremented

  if (processor == 0 )
    return 0;

  if (processor->acquire_leadership() !=1)
    return 0;

  thr_dispatcher.link (this->leaders_);
  thr_dispatcher.processor_ = processor;
    
  int retval = processor->handle_aio_events (wait_time);
    
  thr_dispatcher.processor_ = 0;
  thr_dispatcher.unlink ();

  processor->release_leadership ();
  
  this->promote_leader (processor);
  
  this->process_result_queue (thr_dispatcher);
  
  //ACE_DEBUG((LM_DEBUG,"(%t) ha ret=%d\n", retval));

  return (retval >= 0) ? 1 : -1;
}

int 
ACE_POSIX_AIO_Dispatcher::wait_as_follower(ACE_Time_Value * wait_time,
                                          Thread_Dispatcher & thr_dispatcher)
{
  // Return codes
  // -1   errors
  //  0   wake up event
  //  1   timeout


  int ret_wait = 0;

  if (wait_time == 0)   //  wait forever
    {
      thr_dispatcher.link (this->followers_);

      ret_wait = thr_dispatcher.condition_.wait ();

      thr_dispatcher.unlink ();
    }
  else if ( *wait_time == ACE_Time_Value::zero )
    {
       return 1;
    }
  else
    {
      //Auto decrement wait time
      ACE_Countdown_Time countdown (wait_time);

      ACE_Time_Value abs_time = ACE_OS::gettimeofday ();

      abs_time += (*wait_time);

      thr_dispatcher.link (this->followers_);

      ret_wait = thr_dispatcher.condition_.wait (&abs_time);

      thr_dispatcher.unlink ();
    }

    if (ret_wait == 0)
      return 0;

    if (ACE_OS::last_error() == ETIME)
      return 1;

    ACE_ERROR ((LM_ERROR, 
      ACE_LIB_TEXT("(%t) ACE_POSIX_AIO_Dispatcher::wait_as_follower:")
      ACE_LIB_TEXT(" wait failed\n")));

    return -1;
}

int 
ACE_POSIX_AIO_Dispatcher::handle_events (ACE_Time_Value * wait_time)
{
  int retval = 0;

  int i = 0;

  {
    ACE_Auto_IncDec<size_t> inc_dec_active_threads (this->num_active_threads_);

    
    Thread_Dispatcher * thr_dispatcher = this->allocate_thr_dispatcher();

    if (thr_dispatcher == 0)
      ACE_ERROR_RETURN ((LM_ERROR, 
        ACE_LIB_TEXT("(%t) ACE_POSIX_AIO_Dispatcher::handle_events:")
        ACE_LIB_TEXT(" unable to allocate Thread_Dispatcher\n")),
        -1);

    thr_dispatcher->num_events_ = 0;

    for (;;++i)
      {
        retval = this->process_result_queue (*thr_dispatcher);
        if (retval != 0)  // we have some processed events or errors 
          break;

        //if (i > 0)        // don't go through this point twice
        //  break;

        retval = this->handle_aio_events (wait_time,
                                          this->prime_processor_,
                                          *thr_dispatcher);
        if (retval != 0)  // we have some processed events or errors 
          break;

        retval = this->handle_aio_events (wait_time,
                                          this->second_processor_,
                                          *thr_dispatcher);
        if (retval != 0)  // we have some processed events or errors 
          break;

        retval = this->wait_as_follower (wait_time,
                                         *thr_dispatcher);
        if (retval != 0)  // Timeout or errors 
          break;
      }

      if (retval >= 0) 
        retval = ACE_static_cast (int, thr_dispatcher->num_events_);

      this->free_thr_dispatcher (thr_dispatcher);
  }

  this->promote_leader (this->prime_processor_);
  this->promote_leader (this->second_processor_);
  

  //ACE_DEBUG((LM_DEBUG,"(%t) he ret=%d\n", retval));
  
  return retval;
}

void
ACE_POSIX_AIO_Dispatcher::promote_leader (ACE_POSIX_AIO_Processor * processor)
{
  if (this->followers_.size() != 0)
    {
      if ((processor != 0) &&
          !(processor->has_leader()) &&
          !(processor->is_dedicated_leadership()) &&
          (processor->is_ready_for_leadership()))
        {
          this->notify_follower ();  // let one of followers to be a leader
        }

    }
}

#endif /* ACE_HAS_AIO_CALLS */
