/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
//=============================================================================
/**
 *  @file    POSIX_AIO_LINUX_Provider.h
 *
 *  $Id:  $
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
//=============================================================================

#ifndef ACE_POSIX_AIO_LINUX_PROVIDER_H
#define ACE_POSIX_AIO_LINUX_PROVIDER_H

#include "ace/POSIX_AIO_Provider.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#if defined(ACE_HAS_AIO_CALLS) && defined (ACE_HAS_LINUX_LIBAIO) && defined(linux)



//Forward declarations
 
class ACE_POSIX_AIO_LINUX_Provider;

/**
 * @class ACE_POSIX_AIO_LINUX_Strategy
 *
 */
class ACE_Export ACE_POSIX_AIO_LINUX_Strategy : public ACE_POSIX_AIO_Wait_Strategy
{
  friend class ACE_POSIX_AIO_LINUX_Provider;

protected:
  
  ACE_POSIX_AIO_LINUX_Strategy (ACE_POSIX_AIO_LINUX_Provider & provider);
  virtual ~ACE_POSIX_AIO_LINUX_Strategy();


  // Wait Strategy Interface

  // use base class
  // virtual ACE_POSIX_AIO_Interrupt_Strategy * create_interrupt_strategy (void);

  // use base class
  // virtual void 
  //  destroy_interrupt_strategy (ACE_POSIX_AIO_Interrupt_Strategy * i_strategy);

  virtual int  on_aio_start (ACE_POSIX_Asynch_Result * result);

  virtual int  prepare_to_wait (ACE_POSIX_Asynch_Waiting_List & started_list);

  virtual int  wait (ACE_Time_Value * wait_time);

  virtual ACE_POSIX_Asynch_Result * 
    get_next_completed_result (ACE_POSIX_Asynch_Waiting_List & started_list);

private:

  int  wait_i (timeval * timeout);

  ACE_POSIX_AIO_LINUX_Provider & linux_provider_;
  
  io_event       events_ [ACE_AIO_MAX_SIZE];
  io_event *     next_event_;
  size_t         max_num_events_;
  size_t         num_events_;
};

/**
 * @class ACE_POSIX_AIO_LINUX_Provider
 *
 */
class ACE_Export ACE_POSIX_AIO_LINUX_Provider : public ACE_POSIX_AIO_Provider
{
public:

  friend class ACE_POSIX_AIO_LINUX_Strategy;
  
  ACE_POSIX_AIO_LINUX_Provider (const ACE_POSIX_AIO_Config & config);

  virtual ~ACE_POSIX_AIO_LINUX_Provider();

  virtual int register_handle (ACE_HANDLE handle,const void *completion_key);
  virtual int unregister_handle (ACE_HANDLE handle);

  virtual int get_supported_operations_mask () const;
  virtual int should_start_interrupt_leader () const; 
  virtual int can_start_in_any_thread () const;
  virtual size_t get_max_aio_num () const;

protected:

  /// Provider Interface

  virtual ACE_POSIX_AIO_Wait_Strategy * create_wait_strategy (void);
  virtual void  destroy_wait_strategy (ACE_POSIX_AIO_Wait_Strategy * w_strategy);


  virtual int  start_read_stream  (ACE_POSIX_Asynch_Result * result);
  virtual int  start_write_stream (ACE_POSIX_Asynch_Result * result);
  virtual int  start_read_file    (ACE_POSIX_Asynch_Result * result);
  virtual int  start_write_file   (ACE_POSIX_Asynch_Result * result);
  virtual int  start_read_dgram   (ACE_POSIX_Asynch_Result * result);
  virtual int  start_write_dgram  (ACE_POSIX_Asynch_Result * result);
  virtual int  start_accept       (ACE_POSIX_Asynch_Result * result);
  virtual int  start_connect      (ACE_POSIX_Asynch_Result * result);

  virtual int  cancel_aio (ACE_POSIX_Asynch_Result * result);
  
private :
  io_context_t  context_;
  
};


#endif /* ACE_HAS_AIO_CALLS && ACE_HAS_LINUX_LIBAIO && linux */

#endif /* ACE_POSIX_AIO_LINUX_PROVIDER_H */
