/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */

//=============================================================================
/**
 *
 *  @file    POSIX_AIO_Processor.h
 *
 *  $Id: $
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
//=============================================================================

#ifndef ACE_POSIX_AIO_PROCESSOR_H
#define ACE_POSIX_AIO_PROCESSOR_H

#include "ace/config-all.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#if defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)

#include "ace/Synch_T.h"
#include "ace/POSIX_AIO_Strategies.h"

//Forward declaration
class ACE_POSIX_AIO_Provider;
class ACE_POSIX_AIO_Dispatcher;


/**
 * @class ACE_POSIX_AIO_Processor
 *
 * @brief POSIX implementation .
 */
class ACE_Export ACE_POSIX_AIO_Processor
{
public:
  
  enum Interrupt_Reason
    {
      IR_NEW_STARTED_AIO  = 0x0001,
      IR_NEW_DEFERRED_AIO = 0x0002,
      IR_CANCEL_AIO       = 0x0004,
      IR_BREAK            = 0x0008
    };
  
  /// Constructor.
  ACE_POSIX_AIO_Processor (ACE_POSIX_AIO_Dispatcher & completion_dispatcher,
                           const ACE_POSIX_AIO_Config & config);
  /// Virtual destructor.
  virtual ~ACE_POSIX_AIO_Processor (void);
  
  const ACE_POSIX_AIO_Config & config(void) const;

  int dump (void);

  int create_provider (void);

  int destroy_provider (void);

  int init_provider (void);

  int fini_provider (void);

  virtual int start(void);
  
  virtual int stop(void);

  virtual int is_dedicated_leadership (void) const;

  virtual int is_ready_for_leadership (void) const;

  int  is_supported (int op) const;

  size_t num_total_aios (void) const;

  int has_leader(void) const;

  int has_shared_leader(void) const;

  int has_dedicated_leader(void) const;

  int is_in_leader_context(void) const;

  int is_in_shared_leader_context(void) const;

  int is_in_dedicated_leader_context (void) const;

  int acquire_leadership (void);

  int release_leadership (void);

  int interrupt_leader (int reason);

  int handle_aio_events (ACE_Time_Value * wait_time);

  int start_aio (ACE_POSIX_Asynch_Result * result);

  int start_special_aio (ACE_POSIX_Asynch_Result * result);

  /**
  * This method should be called from
  * ACE_POSIX_Asynch_Operation::cancel()
  * instead of usual ::aio_cancel.
  * For all deferred AIO requests with handle "h"
  * it removes its from the lists and notifies user.
  * For all running AIO requests with handle "h"
  * it calls ::aio_cancel. According to the POSIX standards
  * we will receive ECANCELED  for all ::aio_canceled AIO requests
  * later on return from ::aio_suspend.
  *
  */
  int cancel_aio (ACE_HANDLE h);
  int cancel_aio (ACE_Handler * hanlder);
  int cancel_all_aio(void);

protected:

  virtual int activate_leader (int reason);


  int handle_aio_events_i (ACE_Time_Value * wait_time);

  int post_result (ACE_POSIX_Asynch_Result * result, int notify_flg);

  int poll_events (void);

  int start_deferred_aio (void);

  void report_start_error_i (ACE_POSIX_Asynch_Result *result);

  int cancel_all_deferred_aio (void);

  int cancel_deferred_aio (ACE_HANDLE handle);

  int cancel_deferred_aio (ACE_Handler * handler);

  int cancel_deferred_aio (ACE_POSIX_Asynch_Result * result);

  int cancel_all_started_aio (void);

  int cancel_started_aio (ACE_HANDLE handle);
  
  int cancel_started_aio (ACE_Handler * handler);

  int cancel_started_aio (ACE_POSIX_Asynch_Result * result);

  int interrupt_leader_and_wait (void);
  
  ///  Data members
  /// Mutex to protect work with lists.
  const ACE_POSIX_AIO_Config & config_;

  ACE_POSIX_AIO_Dispatcher &  completion_dispatcher_;

  ACE_SYNCH_MUTEX & mutex_;

  ACE_SYNCH_CONDITION cancel_condition_;

  ACE_POSIX_AIO_Provider * aio_provider_;

  ACE_POSIX_AIO_Wait_Strategy * wait_strategy_;

  ACE_POSIX_AIO_Interrupt_Strategy * interrupt_strategy_;

  
  int       interrupt_state_;
  
  /// List of started AIOs
  ACE_POSIX_Asynch_Waiting_List started_list_;

  /// List of deferred AIOs
  /// Users AIOs are pushed back ( in tail )
  /// Internal (Pipe_Interrupt_Strategy) AIOs are inserted in the head
  ACE_POSIX_Asynch_Result_List deferred_list_;

  ///
  ACE_thread_t leader_id_;
};

#endif /* ACE_HAS_AIO_CALLS */

#endif /* ACE_POSIX_AIO_PROCESSOR_H */
