/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */

//=============================================================================
/**
 *  @file    POSIX_AIO_SGI_Strategy.h
 *
 *  $Id:  $
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
//=============================================================================

#ifndef ACE_POSIX_AIO_SGI_STRATEGY_H
#define ACE_POSIX_AIO_SGI_STRATEGY_H

#include "ace/POSIX_AIO_Strategies.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#if (defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)) && !defined(sun) && !defined(__Lynx__)

#if defined (__xlC__) || defined (__IBMCPP__)

/* AIX does not define sigval_t */
#include <sys/signal.h>
typedef union sigval sigval_t;

#endif // (__xlC__) || defined (__IBMCPP__)



//Forward declarations
class ACE_POSIX_AIO_STD_Provider;

/**
 * @class ACE_POSIX_AIO_SGI_Strategy
 * This class implements both Wait and Interrupt strategies,
 * In previous version it called POSIX_CB_Proactor
 */
class ACE_Export ACE_POSIX_AIO_SGI_Strategy
  : public ACE_POSIX_AIO_Wait_Strategy,
    public ACE_POSIX_AIO_Interrupt_Strategy
{
public:

  ACE_POSIX_AIO_SGI_Strategy (ACE_POSIX_AIO_STD_Provider & provider);

  virtual ~ACE_POSIX_AIO_SGI_Strategy (void);

protected:

  // Wait Strategy Interface
  virtual ACE_POSIX_AIO_Interrupt_Strategy *   create_interrupt_strategy (void);
  
  virtual void 
    destroy_interrupt_strategy (ACE_POSIX_AIO_Interrupt_Strategy * i_strategy);
 
  virtual int on_aio_start (ACE_POSIX_Asynch_Result * result);

  virtual int prepare_to_wait (ACE_POSIX_Asynch_Waiting_List & started_list);

  virtual int wait (ACE_Time_Value * wait_time);

  virtual ACE_POSIX_Asynch_Result * 
    get_next_completed_result (ACE_POSIX_Asynch_Waiting_List & started_list);

  // Interrupt Strategy Interface
  virtual  int init (void);
  virtual  int fini (void);
  virtual  int on_leader_activated (ACE_POSIX_AIO_Processor & aio_processor);
  virtual  int on_leader_deactivated (ACE_POSIX_AIO_Processor & aio_processor);
  virtual  int interrupt(ACE_thread_t leader_id);


  static void  aio_completion_func (sigval_t cb_data);

  ACE_POSIX_AIO_STD_Provider & std_provider_;

  ACE_SYNCH_SEMAPHORE sema_;
};

#endif /* ACE_HAS_AIO_CALLS */

#endif /* ACE_POSIX_AIO_SGI_STRATEGY_H */
