/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
// $Id:  $

#include "ace/POSIX_AIO_STD_Provider.h"

#if defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)

#include "ace/Log_Msg.h"
#include "ace/POSIX_AIO_SGI_Strategy.h"

/**
 *****************************************************************
 *  POSIX Standard AIO Strategy
 *  based on POSIX AIO API
 *
 *****************************************************************
 */
ACE_POSIX_AIO_STD_Strategy::~ACE_POSIX_AIO_STD_Strategy()
{
  ;
}

ACE_POSIX_AIO_STD_Strategy::ACE_POSIX_AIO_STD_Strategy(ACE_POSIX_AIO_STD_Provider & provider)
  : ACE_POSIX_AIO_Wait_Strategy (provider),
    std_provider_ (provider),
    max_num_entries_ (sizeof(aiocb_array_)/sizeof(aiocb_array_[0])),
    num_entries_ (0)
{
  size_t i=0;

  for (; i < this->max_num_entries_;  ++i)
    {
      this->aiocb_array_[i] = 0;
    }

}


int ACE_POSIX_AIO_STD_Strategy::on_aio_start (ACE_POSIX_Asynch_Result * result)
{
  result->aio_sigevent.sigev_notify = SIGEV_NONE;
  return 0; 
}


int
ACE_POSIX_AIO_STD_Strategy::prepare_to_wait(ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_TRACE ("ACE_POSIX_AIO_STD_Strategy::prepare_to_wait");
  
  ACE_POSIX_Asynch_Result * result=0;

  this->num_entries_=0;

  for ( result = started_list.get_first ();
        result != 0;
        result = started_list.get_next ())
    {
      if (this->num_entries_ >= this->max_num_entries_)
        break;

      this->aiocb_array_[this->num_entries_] = result;

      ++this->num_entries_;
    }

  started_list.get_first ();

  if (this->num_entries_ > 0) 
     return WAIT_ENABLED;  // ready to wait
  
  return WAIT_COMPLETED;
}

int
ACE_POSIX_AIO_STD_Strategy::wait (ACE_Time_Value * wait_time)
{
  ACE_TRACE ("ACE_POSIX_AIO_STD_Provider::wait");


  int ret_wait=0;


  if ( wait_time == 0 ) //  wait indefinetely
    {
      ret_wait = ::aio_suspend (this->aiocb_array_,
                                this->num_entries_,
                                0);
    }
  else
    {
      // Block on <aio_suspend> for <milli_seconds>
      timespec timeout = wait_time->operator timespec_t();

      ret_wait = ::aio_suspend (this->aiocb_array_,
                                this->num_entries_,
                                &timeout);
    }

  // Check for errors
  if (ret_wait == -1)
    {
      switch (errno)
        {
        case EAGAIN :   // Timeout
          return WAIT_TIMEOUT;

        case EINTR :    // Interrupted call
          return WAIT_INTERRUPTED;
          
        default:
          ACE_ERROR((LM_ERROR,
            ACE_LIB_TEXT ("%N:%l:(%P | %t)::%p\n"),
            ACE_LIB_TEXT ("ACE_POSIX_AIO_STD_Provider::wait")
            ACE_LIB_TEXT (" <aio_suspend> failed\n")));
          return WAIT_ERROR;

        } //switch

    } //if


  return WAIT_COMPLETED;
}

ACE_POSIX_Asynch_Result *
ACE_POSIX_AIO_STD_Strategy::get_next_completed_result 
  (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  return this->std_provider_.get_next_completed_result (started_list);
}


/**
 *****************************************************************
 *  POSIX Standard AIO Provider
 *  based on POSIX AIO API
 *
 *****************************************************************
 */
ACE_POSIX_AIO_STD_Provider::~ACE_POSIX_AIO_STD_Provider()
{
  ;
}

ACE_POSIX_AIO_STD_Provider::ACE_POSIX_AIO_STD_Provider (const ACE_POSIX_AIO_Config & config)
  : ACE_POSIX_AIO_Provider (config)
{
  this->max_aio_num_ = 
    ACE_POSIX_AIO_Provider::get_os_max_aio_num (config.max_num_aio());
}

int
ACE_POSIX_AIO_STD_Provider::get_supported_operations_mask () const
{
  return (ACE_Asynch_Result_Impl::READ_STREAM 
         |ACE_Asynch_Result_Impl::WRITE_STREAM 
         |ACE_Asynch_Result_Impl::READ_FILE 
         |ACE_Asynch_Result_Impl::WRITE_FILE);
}  
  
 
int 
ACE_POSIX_AIO_STD_Provider::should_start_interrupt_leader () const
{
  // We have to interrupt aio_suspend ()
  // when we start new aio
  // return 1;
  return ACE_POSIX_AIO_Provider::should_start_interrupt_leader ();
}   

int ACE_POSIX_AIO_STD_Provider::can_start_in_any_thread () const
{
  // POSIX API allows start aio from any thread
  //  return 1;
  return ACE_POSIX_AIO_Provider::can_start_in_any_thread ();
}   

size_t
ACE_POSIX_AIO_STD_Provider::get_max_aio_num () const
{
  return this->max_aio_num_; // base class
}

ACE_POSIX_AIO_Wait_Strategy * 
ACE_POSIX_AIO_STD_Provider::create_wait_strategy (void)
{

  ACE_POSIX_AIO_Wait_Strategy * wait_strategy = 0;

  switch (this->config_.ws_type ())
    {

#if !defined(sun) && !defined(__Lynx__)

    case ACE_POSIX_AIO_Config::WST_SGI:
      ACE_NEW_RETURN (wait_strategy,
                      ACE_POSIX_AIO_SGI_Strategy (*this),
                      0);
      break;

#endif

    case ACE_POSIX_AIO_Config::WST_NATIVE:
    default:
      ACE_NEW_RETURN (wait_strategy,
                      ACE_POSIX_AIO_STD_Strategy (*this),
                      0);
      break;
    }

  return wait_strategy;
}

void
ACE_POSIX_AIO_STD_Provider::destroy_wait_strategy (ACE_POSIX_AIO_Wait_Strategy * w_strategy)
{
  delete  w_strategy;
}

int
ACE_POSIX_AIO_STD_Provider::register_handle (ACE_HANDLE handle,
                                             const void *completion_key)
{
  ACE_UNUSED_ARG (handle);
  ACE_UNUSED_ARG (completion_key);

  return 0;
}

int
ACE_POSIX_AIO_STD_Provider::unregister_handle (ACE_HANDLE handle)
{
  ACE_UNUSED_ARG (handle);

  return 0;
}

  
int
ACE_POSIX_AIO_STD_Provider::start_read_stream (ACE_POSIX_Asynch_Result *result)
{  
  int rc = ::aio_read (result);

  this->check_start_error (result, rc);

  return this->should_start_interrupt_leader ();
}

int
ACE_POSIX_AIO_STD_Provider::start_write_stream (ACE_POSIX_Asynch_Result *result)
{  
  int rc = ::aio_write (result);

  this->check_start_error (result,rc);

  return this->should_start_interrupt_leader ();
}

int
ACE_POSIX_AIO_STD_Provider::start_read_file (ACE_POSIX_Asynch_Result *result)
{  
  return this->start_read_stream (result);
}

int
ACE_POSIX_AIO_STD_Provider::start_write_file (ACE_POSIX_Asynch_Result *result)
{  
  return this->start_write_stream (result);
}

int
ACE_POSIX_AIO_STD_Provider::start_read_dgram (ACE_POSIX_Asynch_Result *result)
{
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_STD_Provider::start_write_dgram (ACE_POSIX_Asynch_Result *result)
{  
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_STD_Provider::start_accept (ACE_POSIX_Asynch_Result *result)
{
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_STD_Provider::start_connect (ACE_POSIX_Asynch_Result *result)
{  
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_STD_Provider::cancel_aio (ACE_POSIX_Asynch_Result * result)
{
  /// Cancel pending AIO request
  
  ACE_TRACE ("ACE_POSIX_AIO_STD_Provider::cancel_aio");

  int rc = ::aio_cancel (0, result);

  // may be better ::aio_cancel ( result->aio_fildes, result); 
  // we will cancel all operations 
  // associated with result->aio_fildes,
  // but we can help some OS to cancel


  // Check the return value and translate into enumeration values
  switch (rc)
    {
    case AIO_CANCELED:    return CR_CANCELED;
    case AIO_ALLDONE:     return CR_ALL_DONE;
    case AIO_NOTCANCELED: return CR_NOT_CANCELED;
    default:              break;
    }

  ACE_ERROR ((LM_ERROR,
    ACE_LIB_TEXT ("%N:%l:(%P | %t) ACE_POSIX_AIO_STD_Provider::")
    ACE_LIB_TEXT ("cancel_aio: <aio_cancel> %p\n"),
    ACE_LIB_TEXT ("Unexpected result")));
  
  return CR_CANCEL_ERROR;
}
  
ACE_POSIX_Asynch_Result *
ACE_POSIX_AIO_STD_Provider::get_next_completed_result 
  (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_POSIX_Asynch_Result * result;

  while ((result=started_list.get_current_and_advance()) != 0)
    {
      if (this->test_result_status (result))
        return result;
    }

  return 0;
}

int
ACE_POSIX_AIO_STD_Provider::test_result_status (ACE_POSIX_Asynch_Result *result)
{
  ACE_TRACE ("ACE_POSIX_AIO_STD_Provider::check_result_status");

  // Get the error status of the aio_ operation.
  // aio_error can be called many times
  // while operation status is EINPROGRESS

  int val_aio_error = ::aio_error (result);

  if (val_aio_error == EINPROGRESS)
    return 0;  // not completed

  // aio_return can be called only once
  // it tells OS that result is dequed from OS AIO queue

  ssize_t val_aio_return = ::aio_return (result);

  result->set_ready (val_aio_return, val_aio_error, 0);

  return 1; // completed and marked in ACE_POSIX_Asynch_Result
}


#endif /* ACE_HAS_AIO_CALLS */
