/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
// $Id:  $

#include "ace/POSIX_AIO_SUN_Provider.h"

#if defined(ACE_HAS_AIO_CALLS) && defined(sun)

#include "ace/Log_Msg.h"

/**
 ************************************************************************
 *  SUN AIO Strategy
 *  based on native SUN AIO API
 *
 ************************************************************************
 */

ACE_POSIX_AIO_SUN_Strategy::~ACE_POSIX_AIO_SUN_Strategy (void)
{
}

ACE_POSIX_AIO_SUN_Strategy::ACE_POSIX_AIO_SUN_Strategy (ACE_POSIX_AIO_SUN_Provider & provider)
  : ACE_POSIX_AIO_Wait_Strategy (provider),
    sun_provider_ (provider),
    last_sun_result_(0)
{
}


int ACE_POSIX_AIO_SUN_Strategy::on_aio_start (ACE_POSIX_Asynch_Result * result)
{
  ACE_UNUSED_ARG (result);
  return 0; // do not need to modify aiocb.aio_sigevent
}


int
ACE_POSIX_AIO_SUN_Strategy::prepare_to_wait(ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_TRACE ("ACE_POSIX_AIO_SUN_Strategy::prepare_to_wait");

  ACE_UNUSED_ARG (started_list);

  if (this->last_sun_result_ !=0)
    return WAIT_COMPLETED;

  return WAIT_ENABLED;
}

int
ACE_POSIX_AIO_SUN_Strategy::wait (ACE_Time_Value * wait_time)
{
  timeval timeout ;
  timeval * ptime = 0;
  
  if (wait_time != 0) 
    {
      timeout = wait_time->operator timeval();
      ptime = &timeout;
    }
  
  return this->wait_i (ptime);
}

int
ACE_POSIX_AIO_SUN_Strategy::wait_i (timeval * ptime)
{
  ACE_TRACE ("ACE_POSIX_AIO_SUN_Strategy::wait_i");

  if (this->last_sun_result_ != 0)
    return WAIT_COMPLETED;

  this->last_sun_result_ = ::aiowait (ptime);

  // Check for errors
  if (this->last_sun_result_ == 0)    // timeout
    {
       errno = EAGAIN;
       return WAIT_TIMEOUT;
    }

  if (ACE_reinterpret_cast (long, this->last_sun_result_) == -1 )
    {
      this->last_sun_result_ = 0;

      // Check errno  for  EINVAL,EAGAIN,EINTR ??
      switch (errno)
        {
        case EINTR :              // aiowait() was interrupted by a signal.
          return WAIT_INTERRUPTED;

        case EINVAL:              // there are no outstanding AIO requests.
          errno = EAGAIN;         // make POSIX-identical errno
          return WAIT_TIMEOUT;      

        default:         // EFAULT
          ACE_ERROR ((LM_ERROR,
            ACE_LIB_TEXT ("%N:%l:(%P | %t)::%p\n"),
            ACE_LIB_TEXT ("ACE_POSIX_AIO_SUN_Strategy::")
            ACE_LIB_TEXT ("wait: <aiowait> failed")));

          return WAIT_ERROR;
       }
    }

  return WAIT_COMPLETED;
}

ACE_POSIX_Asynch_Result *
ACE_POSIX_AIO_SUN_Strategy::get_next_completed_result 
  (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  if (this->last_sun_result_ == 0)
    {
      timeval timeout = {0,0};

      this->wait_i (&timeout);
    }

  if (this->last_sun_result_ == 0)
    return 0;

  
  ACE_POSIX_Asynch_Result * result;

  for ( result = started_list.get_first();
        result != 0;
        result = started_list.get_next ())
    {
      if (this->last_sun_result_ == & result->aio_resultp)
        {
          this->last_sun_result_ =0;

          this->sun_provider_.test_result_status (result);

          return result;
        }
    }

  // somebody except us uses native aioXXXX api!!!
  
  ACE_ERROR ((LM_ERROR,
            ACE_LIB_TEXT ("%N:%l:(%P | %t)::%p\n"),
            ACE_LIB_TEXT ("ACE_POSIX_AIO_SUN_Strategy::")
            ACE_LIB_TEXT ("unknown aio_result_t")));


  this->last_sun_result_ =0;

  return 0;
}

/**
 ************************************************************************
 *  SUN AIO Provider
 *  based on native SUN AIO API
 *
 ************************************************************************
 */


ACE_POSIX_AIO_SUN_Provider::~ACE_POSIX_AIO_SUN_Provider (void)
{
}

ACE_POSIX_AIO_SUN_Provider::ACE_POSIX_AIO_SUN_Provider (const ACE_POSIX_AIO_Config & config)
  : ACE_POSIX_AIO_Provider (config)
{
  this->max_aio_num_ = 
    ACE_POSIX_AIO_Provider::get_os_max_aio_num (config.max_num_aio());
}


int
ACE_POSIX_AIO_SUN_Provider::get_supported_operations_mask () const
{
  return (ACE_Asynch_Result_Impl::READ_STREAM 
         |ACE_Asynch_Result_Impl::WRITE_STREAM 
         |ACE_Asynch_Result_Impl::READ_FILE 
         |ACE_Asynch_Result_Impl::WRITE_FILE);
}  
  
 
int 
ACE_POSIX_AIO_SUN_Provider::should_start_interrupt_leader () const
{
  // We do not have to inrterrupt aiowait() 
  // when we start new aio
  // return 0;
  return ACE_POSIX_AIO_Provider::should_start_interrupt_leader ();
}   

int ACE_POSIX_AIO_SUN_Provider::can_start_in_any_thread () const
{
  // Sun native API should allow start aio from any thread
  // return 1;
  return ACE_POSIX_AIO_Provider::can_start_in_any_thread ();
}   

size_t
ACE_POSIX_AIO_SUN_Provider::get_max_aio_num () const
{
  return this->max_aio_num_; // base class
}

ACE_POSIX_AIO_Wait_Strategy * 
ACE_POSIX_AIO_SUN_Provider::create_wait_strategy (void)
{
  ACE_POSIX_AIO_Wait_Strategy * wait_strategy = 0;

  ACE_NEW_RETURN (wait_strategy,
                  ACE_POSIX_AIO_SUN_Strategy (*this),
                  0);

  return wait_strategy;
}

void
ACE_POSIX_AIO_SUN_Provider::destroy_wait_strategy (ACE_POSIX_AIO_Wait_Strategy * w_strategy)
{
  delete  w_strategy;
}

int
ACE_POSIX_AIO_SUN_Provider::register_handle (ACE_HANDLE handle,
                                             const void *completion_key)
{
  ACE_UNUSED_ARG (handle);
  ACE_UNUSED_ARG (completion_key);

  return 0;
}

int
ACE_POSIX_AIO_SUN_Provider::unregister_handle (ACE_HANDLE handle)
{
  ACE_UNUSED_ARG (handle);

  return 0;
}

int
ACE_POSIX_AIO_SUN_Provider::start_read_stream (ACE_POSIX_Asynch_Result *result)
{  
  // ****** from Sun man pages *********************
  // Upon completion of the operation both aio_return and aio_errno
  // are set to reflect the result of the operation.
  // AIO_INPROGRESS is not a value used by the system
  // so the client may detect a change in state
  // by initializing aio_return to this value.

  result->aio_resultp.aio_return = AIO_INPROGRESS;
  result->aio_resultp.aio_errno  = EINPROGRESS;

  int rc =::aioread (result->aio_fildes,
                     (char *) result->aio_buf,
                     result->aio_nbytes,
                     result->aio_offset,
                     SEEK_SET,
                     &result->aio_resultp);

  this->check_start_error (result, rc);

  return this->should_start_interrupt_leader ();
}

int
ACE_POSIX_AIO_SUN_Provider::start_write_stream (ACE_POSIX_Asynch_Result *result)
{  
  // ****** from Sun man pages *********************
  // Upon completion of the operation both aio_return and aio_errno
  // are set to reflect the result of the operation.
  // AIO_INPROGRESS is not a value used by the system
  // so the client may detect a change in state
  // by initializing aio_return to this value.

  result->aio_resultp.aio_return = AIO_INPROGRESS;
  result->aio_resultp.aio_errno  = EINPROGRESS;

  int rc = ::aiowrite (result->aio_fildes,
                       (char *) result->aio_buf,
                        result->aio_nbytes,
                       result->aio_offset,
                       SEEK_SET,
                       &result->aio_resultp);

  this->check_start_error (result, rc);

  return this->should_start_interrupt_leader ();
}

int
ACE_POSIX_AIO_SUN_Provider::start_read_file (ACE_POSIX_Asynch_Result *result)
{  
  return this->start_read_stream (result);
}

int
ACE_POSIX_AIO_SUN_Provider::start_write_file (ACE_POSIX_Asynch_Result *result)
{  
  return this->start_write_stream (result);
}

int
ACE_POSIX_AIO_SUN_Provider::start_read_dgram (ACE_POSIX_Asynch_Result *result)
{
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_SUN_Provider::start_write_dgram (ACE_POSIX_Asynch_Result *result)
{  
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_SUN_Provider::start_accept (ACE_POSIX_Asynch_Result *result)
{
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_SUN_Provider::start_connect (ACE_POSIX_Asynch_Result *result)
{  
  return ACE_POSIX_AIO_Provider::start_not_supported (result);
}

int
ACE_POSIX_AIO_SUN_Provider::cancel_aio (ACE_POSIX_Asynch_Result * result)
{
  /// Cancel pending AIO request

  ACE_TRACE ("ACE_POSIX_AIO_SUN_Provider::cancel_aio");

  int rc = ::aiocancel (&result->aio_resultp);
  
  // Check the return value and translate into enumeration values
  
  // ****** from Sun man pages *********************
  // Upon successful completion, 
  // aiocancel() returns 0 and the requested operation is cancelled.
  // The application will not receive the SIGIO completion signal 
  // for an asynchronous operation that is successfully cancelled.

  if (rc == 0)       // OK, canceled and dequeued
    {
      this->test_result_status (result);
    
      return CR_CANCELED;
    }
  
  switch ( errno )   // ****** from Sun man pages *********************
    {
    case EINVAL :                 // Thee are not any outstanding 
      return CR_ALL_DONE;         // requests to cancel.

    case EACCES :                 // The parameter resultp does not 
      return CR_ALL_DONE;         // correspond to any outstanding 
                                  // asynchronous operation,
                                  // although there is at least 
                                  // one currently outstanding.

    case EFAULT:                  // resultp points to an address 
                                  // outside the address space 
                                  // of the requesting  process.
    default:
      break;
    }

  return CR_CANCEL_ERROR;
}
  
int
ACE_POSIX_AIO_SUN_Provider::test_result_status (ACE_POSIX_Asynch_Result *result)
{
  ACE_TRACE ("ACE_POSIX_AIO_SUN_Provider::test_result_status");

  int  val_aio_error = result->aio_resultp.aio_errno;
  ssize_t val_aio_return = result->aio_resultp.aio_return;

  ACE_ASSERT (val_aio_return != AIO_INPROGRESS);

  result->set_ready (val_aio_return, val_aio_error, 0);

  return 1;
}


#endif /* ACE_HAS_AIO_CALLS && sun */
