/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */

//=============================================================================
/**
 *  @file    POSIX_AIO_Select_Provider.h
 *
 *  $Id:  $
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
//=============================================================================

#ifndef ACE_POSIX_AIO_SELECT_PROVIDER_H
#define ACE_POSIX_AIO_SELECT_PROVIDER_H

#include "ace/POSIX_AIO_Provider.h"

#if defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)

#include "ace/Select_Reactor_Base.h"
#include <poll.h>


#define  IO_READY  ACE_reinterpret_cast(ACE_POSIX_Asynch_Result*,1U)

//Forward declarations

class ACE_POSIX_AIO_Select_Provider;

/**
 * @class ACE_POSIX_AIO_Select_Strategy_Base
 *
 */

class ACE_Export ACE_POSIX_AIO_Select_Strategy_Base : public ACE_POSIX_AIO_Wait_Strategy
{
  friend class ACE_POSIX_AIO_Select_Provider;

protected:

  class Handle_Info 
    {
      friend class ACE_POSIX_AIO_Select_Strategy_Base;

    public:
      Handle_Info (void);
      virtual ~Handle_Info (void);

      int   handle          (void) const;
      int   is_read_ready   (void) const;
      int   is_write_ready  (void) const;
      u_int ready_events    (void) const;
      u_int events_to_clear (void) const;
      u_int events_to_set   (void) const;

      void set (ACE_HANDLE fd, u_int ready_events);

      u_int poll (void);
      
      u_int clear_ready (u_int events);

      u_int request_to_clear (u_int events);
      u_int request_to_set   (u_int events);

      void  set_result(ACE_POSIX_Asynch_Result * result);
      ACE_POSIX_Asynch_Result * get_result(void);

    private:

      ACE_HANDLE fd_;
      u_int      ready_events_;
      u_int      events_to_clear_;
      u_int      events_to_set_;

      ACE_POSIX_Asynch_Result * ready_result_;
    };




  ACE_POSIX_AIO_Select_Strategy_Base (ACE_POSIX_AIO_Select_Provider & provider);
  virtual ~ACE_POSIX_AIO_Select_Strategy_Base (void);

  virtual int declare_interest (ACE_HANDLE handle, u_int mask) =0;
  virtual int clear_interest   (ACE_HANDLE handle, u_int mask) =0;

  int start_read   (ACE_POSIX_Asynch_Result * result);
  int start_write  (ACE_POSIX_Asynch_Result * result);
  int cancel_read  (ACE_POSIX_Asynch_Result * result);
  int cancel_write (ACE_POSIX_Asynch_Result * result);

  int check (Handle_Info & fd_state);
  

  int alloc_repository (size_t size, int flg_ready);
  int free_repository  (void);


  ACE_POSIX_AIO_Select_Provider & sel_provider_;

  ACE_POSIX_Asynch_Result ** pending_reads_;
  ACE_POSIX_Asynch_Result ** pending_writes_;
  size_t size_;

  Handle_Info fd_info_;
};

/**
 * @class ACE_POSIX_AIO_Select_Provider
 *
 */

class ACE_Export ACE_POSIX_AIO_Select_Provider : public ACE_POSIX_AIO_Provider
{
public:
  
  ACE_POSIX_AIO_Select_Provider (const ACE_POSIX_AIO_Config & config);

  virtual ~ACE_POSIX_AIO_Select_Provider();

  virtual int register_handle (ACE_HANDLE handle,const void *completion_key);
  virtual int unregister_handle (ACE_HANDLE handle);

  virtual int get_supported_operations_mask () const;

  virtual int should_start_interrupt_leader () const; 
  virtual int can_start_in_any_thread () const;
  virtual size_t get_max_aio_num () const;

  int do_read   (ACE_POSIX_Asynch_Result * result);
  int do_write  (ACE_POSIX_Asynch_Result * result);

protected:
  virtual ACE_POSIX_AIO_Wait_Strategy * create_wait_strategy (void);
  virtual void  destroy_wait_strategy (ACE_POSIX_AIO_Wait_Strategy * w_strategy);


  virtual int  start_read_stream  (ACE_POSIX_Asynch_Result * result);
  virtual int  start_write_stream (ACE_POSIX_Asynch_Result * result);
  virtual int  start_read_file    (ACE_POSIX_Asynch_Result * result);
  virtual int  start_write_file   (ACE_POSIX_Asynch_Result * result);
  virtual int  start_read_dgram   (ACE_POSIX_Asynch_Result * result);
  virtual int  start_write_dgram  (ACE_POSIX_Asynch_Result * result);
  virtual int  start_accept       (ACE_POSIX_Asynch_Result * result);
  virtual int  start_connect      (ACE_POSIX_Asynch_Result * result);

  virtual int  cancel_aio (ACE_POSIX_Asynch_Result * result);

  int do_read_stream    (ACE_POSIX_Asynch_Result * result);
  int do_write_stream   (ACE_POSIX_Asynch_Result * result);
  int do_read_file      (ACE_POSIX_Asynch_Result * result);
  int do_write_file     (ACE_POSIX_Asynch_Result * result);
  int do_read_dgram     (ACE_POSIX_Asynch_Result * result);
  int do_write_dgram    (ACE_POSIX_Asynch_Result * result);
  int do_accept         (ACE_POSIX_Asynch_Result * result);
  int do_connect        (ACE_POSIX_Asynch_Result * result);


  ACE_POSIX_AIO_Select_Strategy_Base * sel_strategy_;
};

#endif /* ACE_HAS_AIO_CALLS */

#endif /* ACE_POSIX_AIO_SELECT_PROVIDER_H */
