/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

/* -*- C++ -*- */
// $Id:  $

#include "ace/POSIX_AIO_Select_Strategy.h"

#if defined (ACE_HAS_AIO_CALLS) || defined(ACE_HAS_AIO_EMULATION)

#include "ace/Log_Msg.h"
#include "ace/Addr.h"
#include "ace/ACE.h"


/**
 *****************************************************************
 *  POSIX Select-based Wait Strategy
 *
 *****************************************************************
 */
ACE_POSIX_AIO_Select_Strategy::~ACE_POSIX_AIO_Select_Strategy()
{
  ;
}

ACE_POSIX_AIO_Select_Strategy::ACE_POSIX_AIO_Select_Strategy 
  (ACE_POSIX_AIO_Select_Provider & provider)
  : ACE_POSIX_AIO_Select_Strategy_Base (provider),
    hmax_ (ACE_INVALID_HANDLE)
{
}

int 
ACE_POSIX_AIO_Select_Strategy::init (void)
{
  if (this->alloc_repository (ACE_Handle_Set::MAXSIZE, 0) != 0)
    return -1;

  ACE_DEBUG ((LM_DEBUG,
    ACE_LIB_TEXT("(%t)ACE_POSIX_AIO_Select_Strategy size=%d\n"), 
    (int) this->size_));

  return 0; 
}

int 
ACE_POSIX_AIO_Select_Strategy::fini (void)
{
  this->free_repository ();
  return 0; 
}

int
ACE_POSIX_AIO_Select_Strategy::on_aio_start (ACE_POSIX_Asynch_Result * result)
{
  ACE_HANDLE handle = result->aio_fildes;

  int blk_mode = ACE_BIT_DISABLED (ACE::get_flags (handle), ACE_NONBLOCK);

  // Set the handle into non-blocking mode if it's not already in it.
  if (blk_mode)
    ACE::set_flags (handle, ACE_NONBLOCK);

  return 0; 
}

int
ACE_POSIX_AIO_Select_Strategy::prepare_to_wait (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_TRACE ("ACE_POSIX_AIO_Select_Strategy::prepare_to_wait");

  ACE_UNUSED_ARG (started_list);

  if (hmax_ != 0)
    return WAIT_COMPLETED; 

  this->ready_set_ = this->request_set_;
  
  ACE_HANDLE hr = this->ready_set_.rd_mask_.max_set();
  ACE_HANDLE hw = this->ready_set_.wr_mask_.max_set();

  
  this->hmax_ = (hr > hw) ? hr : hw;

  if (this->hmax_ < 0)
   {
//     ACE_DEBUG ((LM_DEBUG,"**** hmax = %d\n", this->hmax_));

     this->hmax_ = 0;
     return WAIT_TIMEOUT; 
   }

  this->hmax_++;
  return WAIT_ENABLED;
}

int
ACE_POSIX_AIO_Select_Strategy::wait (ACE_Time_Value * wait_time)
{
  ACE_TRACE ("ACE_POSIX_AIO_Select_Strategy::wait");


  // I'd like to use ACE_OS::select(),
  // but it re-issued select in case of EINTR.
  // We can use EINTR as signal to stop select. 

  timeval tv;
  timeval *timep =0;

  if (wait_time != 0)
    {
      tv = * wait_time;
      timep = &tv;
    }

//
//  ACE_ERROR((LM_ERROR,"(%t) Before select hmax=%d\n",this->hmax_));
//
//  this->ready_set_.rd_mask_.dump();
//  this->ready_set_.wr_mask_.dump();
//  this->ready_set_.ex_mask_.dump();
  
  int rc =::select ((int)this->hmax_,
                    (ACE_FD_SET_TYPE *) this->ready_set_.rd_mask_,
                    (ACE_FD_SET_TYPE *) this->ready_set_.wr_mask_,
                    (ACE_FD_SET_TYPE *) 0,
                    timep);

//
//  int save_err =errno;
//
//  ACE_ERROR((LM_ERROR,"(%t) After select rc=%d hmax=%d errno=%d %s\n",
//             rc, 
//             this->hmax_,
//              rc >= 0 ? 0 : save_err , 
//              rc >= 0 ? "no error" : ACE_OS::strerror(save_err)));
//
//  this->ready_set_.rd_mask_.dump();
//  this->ready_set_.wr_mask_.dump();
//  this->ready_set_.ex_mask_.dump();
//
//  errno=save_err;
//

  if (rc > 0)
    {
      this->find_next_ready_fd (0);
      return WAIT_COMPLETED;
    }


  if (rc == 0)
    {
      this->hmax_ = 0;
      return WAIT_TIMEOUT;
    }


  switch (errno)
    {
    case EINTR :    // Interrupted call
      this->hmax_ = 0;
      return WAIT_INTERRUPTED;

    case EBADF :  
      //  TODO: some descriptors are invalid.
      //  Reason: user closed file with pending AIO!
      //  Action: check all results for valid descriptors.
      //  If descriptor is invalid, complete AIO
      //  with ECANCELED/EABORTED
      //this->ready_set_ = this->request_set_;
      
      for ( int i=0; i < this->hmax_; ++i)
      {
        this->ready_set_.rd_mask_.set_bit(i);
        this->ready_set_.wr_mask_.set_bit(i);
      }
      
      this->find_next_ready_fd (0);
      return WAIT_COMPLETED;

    default:
      break;
    }
    
  this->hmax_ = 0;
  return WAIT_ERROR;
}

ACE_POSIX_Asynch_Result *
ACE_POSIX_AIO_Select_Strategy::get_next_completed_result 
  (ACE_POSIX_Asynch_Waiting_List & started_list)
{
  ACE_UNUSED_ARG (started_list);

  for (;;)
    {

      ACE_HANDLE handle = this->fd_info_.handle();

      if (handle == ACE_INVALID_HANDLE)
        break;

      int rc = this->check(this->fd_info_);

      if (rc != 0)
        {
          ACE_POSIX_Asynch_Result * result = this->fd_info_.get_result();

          ACE_ASSERT (result !=0);

          return result;
        }

      this->clear_interest  (handle,
                             this->fd_info_.events_to_clear());

      this->declare_interest(handle,
                             this->fd_info_.events_to_set());

//      if (this->pending_writes_[handle] == IO_READY)
//        ACE_ASSERT(this->request_set_.wr_mask_.is_set (handle) == 0);

//      if (this->pending_reads_[handle] == IO_READY)
//        ACE_ASSERT(this->request_set_.rd_mask_.is_set (handle) == 0);

      ++handle;

      this->find_next_ready_fd(handle);
    }
      

  this->hmax_ = 0;

  return 0;
}

int 
ACE_POSIX_AIO_Select_Strategy::find_next_ready_fd (ACE_HANDLE fd)
{
  for (; fd >= 0 &&  fd < this->hmax_; ++fd )
    {
      u_int revents = 0;

      if (this->ready_set_.wr_mask_.is_set (fd))
        revents |= POLLOUT;

      if (this->ready_set_.rd_mask_.is_set (fd))
        revents |= POLLIN;

      if (revents != 0)
        {
          this->fd_info_.set(fd, revents);
          return 1;
        }
    }

  this->fd_info_.set(ACE_INVALID_HANDLE, 1);
  return 0;
}

// *****************************************************************
//  Declare interest
//  Return code :
//    1 - we should interrupt leader to update wait state
//    0 - we do not have to interrupt leader
// *****************************************************************

int
ACE_POSIX_AIO_Select_Strategy::declare_interest (ACE_HANDLE handle, 
                                                 u_int mask)
{  
  int rc = 0;

  if ((mask & POLLIN) == POLLIN)
    {
      if (this->request_set_.rd_mask_.is_set (handle) == 0)
        {
          this->request_set_.rd_mask_.set_bit (handle);
          rc = 1;
        }
    }

  if ((mask & POLLOUT) == POLLOUT)
    {
      if (this->request_set_.wr_mask_.is_set (handle) == 0)
        {
          this->request_set_.wr_mask_.set_bit (handle);
          rc = 1;
        }
    }
        
  return rc;
}

int
ACE_POSIX_AIO_Select_Strategy::clear_interest (ACE_HANDLE handle, 
                                                 u_int mask)
{  
  int rc = 0;

  if ((mask & POLLIN) == POLLIN)
    {
      if (this->request_set_.rd_mask_.is_set (handle))
        {
          this->request_set_.rd_mask_.clr_bit (handle);
          rc = 1;
        }
    }


  if ((mask & POLLOUT) == POLLOUT)
    {
      if (this->request_set_.wr_mask_.is_set (handle))
        {
          this->request_set_.wr_mask_.clr_bit (handle);
          rc = 1;
        }
    }
        
  return rc;
}

#endif /* ACE_HAS_AIO_CALLS */
