/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

// ============================================================================
/**
 *  @file ACE_Asynch_RW.cpp
 *
 *  ACE_Asynch_RW.cpp
 *
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
// ============================================================================
#ifndef ACE_ASYNCH_RW_C
#define ACE_ASYNCH_RW_C

#include "Asynch_RW.h"

template <class READER, class WRITER>
ACE_Asynch_RW<READER, WRITER>::ACE_Asynch_RW (void)
  : handle_ (ACE_INVALID_HANDLE),
    flg_owner_(0),
    flg_cancel_(0),
    completion_key_(0),
    rs_(),
    ws_()
{
  ;
}

template <class READER, class WRITER>
ACE_Asynch_RW<READER, WRITER>::~ACE_Asynch_RW (void)
{
  this->close();
}

template <class READER, class WRITER> int
ACE_Asynch_RW<READER, WRITER>::close(void)
{

  if (this->handle_ != ACE_INVALID_HANDLE && this->flg_owner_ != 0)
    ACE_OS::closesocket (this->handle_);

  this->handle_ = ACE_INVALID_HANDLE;

  return 0;
}

template <class READER, class WRITER> int
ACE_Asynch_RW<READER, WRITER>::cancel(void)
{
  if (this->flg_cancel_ == 0)
    {
      this->flg_cancel_ = 1;
    
      if (this->handle_ != ACE_INVALID_HANDLE) 
        {
           this->rs_.cancel ();
           this->ws_.cancel ();
        }
    }

  return 0;
}

template <class READER, class WRITER> int
ACE_Asynch_RW<READER, WRITER>::cancel_and_close(void)
{
  this->cancel();
  return this->close();
}

template <class READER, class WRITER> int
ACE_Asynch_RW<READER, WRITER>::open 
         (ACE_Handler &handler,
          ACE_HANDLE handle, 
          const void *completion_key,
          ACE_Proactor *proactor,
          int pass_ownership)
{

  if (this->handle_ != ACE_INVALID_HANDLE || this->flg_cancel_ != 0)
    return -1;

  this->handle_ = handle;
  this->completion_key_ = completion_key;
  this->flg_owner_ = pass_ownership;

  if (this->rs_.open(handler,
                     handle,
                     completion_key,
                     proactor) < 0)
  
    return -1;

  if (this->ws_.open(handler,
                     handle,
                     completion_key,
                     proactor) < 0)
  
    return -1;

  return 0;
}

// ****************************************************************

template <class READER, class WRITER> int
ACE_Asynch_RW_Stream<READER, WRITER>::read (ACE_Message_Block &message_block,
                            size_t num_bytes_to_read,
                            const void *act,
                            int priority,
                            int signal_number)
{
  if (this->handle_ == ACE_INVALID_HANDLE || flg_cancel_ != 0)
    return -1;

  return this->reader().read (message_block,
                              num_bytes_to_read,
                              act,
                              priority,
                              signal_number);
}
 
template <class READER, class WRITER> int
ACE_Asynch_RW_Stream<READER, WRITER>::write (ACE_Message_Block &message_block,
                             size_t num_bytes_to_write,
                             const void *act,
                             int priority,
                             int signal_number)
{
  if (this->handle_ == ACE_INVALID_HANDLE || flg_cancel_ != 0)
    return -1;

  return this->writer().write (message_block,
                               num_bytes_to_write,
                               act,
                               priority,
                               signal_number);
}

// ****************************************************************

template <class READER, class WRITER> ssize_t
ACE_Asynch_RW_Dgram<READER, WRITER>::recv (ACE_Message_Block *  message_block,  
                 size_t &             number_of_bytes_recvd,  
                 int                  flags,  
                 int                  protocol_family,
                 const void *         act,
                 int                  priority,
                 int                  signal_number)
{
  if (this->handle_ == ACE_INVALID_HANDLE || flg_cancel_ != 0)
    return -1;

  return this->reader().recv (message_block,
                              number_of_bytes_recvd,  
                              flags,  
                              protocol_family,
                              act,
                              priority,
                              signal_number);
}
 

template <class READER, class WRITER> ssize_t
ACE_Asynch_RW_Dgram<READER, WRITER>::send  (ACE_Message_Block * message_block,  
                 size_t &            number_of_bytes_sent,  
                 int                 flags,  
                 const ACE_Addr &    remote_addr,  
                 const void *        act,  
                 int                 priority,  
                 int                 signal_number)
{
  if (this->handle_ == ACE_INVALID_HANDLE || flg_cancel_ != 0)
    return -1;

  return this->writer().send  (message_block,
                               number_of_bytes_sent,  
                               flags,  
                               remote_addr,  
                               act,  
                               priority,  
                               signal_number);
}


#endif //ACE_ASYNCH_RW_C


