/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

// ============================================================================
//
// = AUTHOR
//    Alexander Libman <libman@terabit.com.au>
//
// ============================================================================

#ifndef ACE_TESTS_CFG_H
#define ACE_TESTS_CFG_H

#include "ace/Service_Config.h"
#include "ace/Get_Opt.h"
#include "ace/streams.h"
#include "ace/Log_Msg.h"

// helper functions
int 
disable_signal (int sigmin, int sigmax);

int
do_delay (u_long millisec);

// Proactor Type (UNIX only, Win32 ignored)
enum ProactorType
  { 
    DEFAULT = 0, 
    AIOCB,
    SIG,
    CB,
    SUN,
    SELECT,
    POLL,
    DEVPOLL,
    EPOLL,
    LINUXRT,
    LINUX,
    DUMMY
  }; 

const  size_t MIN_TIME     = 0;         // min 0 sec
const  size_t MAX_TIME     = 3600;      // max 1 hour
const  u_int  MAX_BLK      = 1024*128;  // 128K
const  u_int  MIN_BLK      = 1;         //
const  u_int  MAX_WIN      = 1024*512;  // 512K
const  u_int  MIN_WIN      = 0;         //
const  size_t MIN_DELAY    = 0;         // min 0
const  size_t MAX_DELAY    = 1000*1000; // 1 sec in microsec
const  u_int  MAX_SESSIONS = 10000;      //
const  u_int  MAX_CONNECTIONS = MAX_SESSIONS;




class LogLocker
{
public:

  LogLocker () { ACE_LOG_MSG->acquire (); }
  virtual ~LogLocker () { ACE_LOG_MSG->release (); }
};

class TestCfg
{
  
public:

  TestCfg  (void);
  virtual ~TestCfg (void);


  int          init ();
  int          parse_args (int argc, ACE_TCHAR *argv[]);
  int          print_usage(int argc, ACE_TCHAR *argv[]);

  void         set_blksize (u_int blksize);
  int          set_proactor_type (const ACE_TCHAR *ptype);



  u_int        threads ()             const { return threads_            ;}
  ProactorType proactor_type ()       const { return proactor_type_      ;}
  size_t       max_aio_operations ()  const { return max_aio_operations_ ;}
  int          sig_num ()             const { return sig_num_            ;}
  int          leader_type ()         const { return leader_type_        ;}  
  int          start_aio_type ()      const { return start_aio_type_     ;}
  int          both ()                const { return both_               ;}
  const ACE_TCHAR * connect_host ()   const { return connect_host_       ;}
  u_short      connect_port ()        const { return connect_port_       ;}
  u_short      listen_port ()         const { return listen_port_        ;}
  u_int        sessions ()            const { return sessions_           ;}
  int          loglevel ()            const { return loglevel_           ;} 
  u_int        seconds ()             const { return seconds_            ;}  
  u_long       delay ()               const { return delay_              ;}    
  u_long       timeout ()             const { return timeout_            ;}
  size_t       xfer_limit ()          const { return xfer_limit_         ;}  
  u_int        blksize ()             const { return blksize_            ;}
  u_int        s_blksize ()           const { return s_blksize_          ;}
  u_int        r_blksize ()           const { return r_blksize_          ;}
  u_int        w_size ()              const { return w_size_             ;}  


protected:

  // Thread Pool Size
  u_int threads_;

  ProactorType proactor_type_;

  // POSIX : > 0 max number aio operations  proactor,
  size_t max_aio_operations_;

  // POSIX : signal to interrupt (0- PIPE Strategy,
  //                              sig_num - SIGNAL Strategy)
  int  sig_num_;

  // POSIX : leader type :  0-shared / 1-dedicated
  int  leader_type_;  

  // POSIX : start aio type :  1 - any thread , 0- only leader
  int  start_aio_type_;

  // both:  0 run client or server 
  //          sessions = 0 server
  //          sessions != 0 client
  //     != 0 run client and server
  int both_;

  // Host that we're connecting to.
  const ACE_TCHAR * connect_host_;
  
  // Port that we're connecting to.
  u_short connect_port_;

  // Port that we're receiving connections on.
  u_short listen_port_;
  
  // Clients sessions
  u_int sessions_;

  // Log options
  int   loglevel_; // 0 full , 1 medium, 2 only errors

  // seconds to run - 0 means prompt a message to stop
  u_int seconds_;  // default time to run - 2 seconds

  // delay - emulate long callback
  u_long delay_;    

  // timeout per operation (evictor pattern)
  u_long timeout_;


  // Number of bytes for Sender to send. 
  size_t xfer_limit_;  

  u_int  blksize_;
  u_int  s_blksize_;
  u_int  r_blksize_;
  u_int  w_size_;
};

#endif // ACE_TESTS_CFG_H
