/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

// ============================================================================
/**
 *  @file Receiver.h
 *
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
// ============================================================================
#ifndef SSL_CONNECTION_H
#define SSL_CONNECTION_H

#include "PConnection.h"
#include "Asynch_RW.h"

#if defined (ACE_HAS_THREADS) && \
    ((defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)) || (defined (ACE_HAS_AIO_CALLS)))
// This only works on Win32 platforms and on Unix platforms
// supporting POSIX aio calls.


#include "ace/SSL/sslconf.h"
#include "ace/SSL/SSL_Asynch_Stream.h"
#include "openssl/rand.h"



// *************************************************************
//   Receiver SSL
// *************************************************************

class SSLConnection : public PConnection 
{
public:
  SSLConnection  (void);
  ~SSLConnection (void);

  // Virtual from PConnection
  virtual const ACE_TCHAR * get_name (void) const 
    {
       return ACE_TEXT("Receiver");
    }

  virtual void cancel ();
  
  virtual int  is_safe_to_delete (void);

  virtual void on_open (ACE_HANDLE handle);

  virtual int  start_asynch_read (ACE_Message_Block &mb, size_t bytes);
    
  virtual int  start_asynch_write (ACE_Message_Block &mb, size_t bytes);

  /// This is called after all I/O is done and
  /// it is safe to self-destroy 
  virtual void handle_wakeup ();

private:

  ACE_SSL_Asynch_Stream stream_;
  int flg_cancel_;
  int flg_safe_delete_;
};

#endif  /* ACE_WIN32 && !ACE_HAS_WINCE || ACE_HAS_AIO_CALLS */
#endif  /* SSL_CONNECTION_H */

