/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/

// ============================================================================
/**
 *  @file SSLSession.cpp
 *
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
// ============================================================================

#include "SSLSession.h"

#if defined (ACE_HAS_THREADS) && \
    ((defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)) || (defined (ACE_HAS_AIO_CALLS)))
// This only works on Win32 platforms and on Unix platforms
// supporting POSIX aio calls.


// ***************************************************
//
// ***************************************************
SSLSession::SSLSession ()
: PSession         (),
  stream_          (ACE_SSL_Asynch_Stream::ST_SERVER),
  flg_cancel_      (0),
  flg_safe_delete_ (1)

{
}

SSLSession::~SSLSession (void)
{
}

void
SSLSession::cancel ()
{
  this->flg_cancel_ = 1;
  this->stream_.cancel ();

}

int 
SSLSession::is_safe_to_delete (void)
{
  if (this->has_pending_io ())
    return 0;

  if (flg_safe_delete_)
    return 1;

  this->stream_.close();
  return 0;
}

void
SSLSession::on_open (ACE_HANDLE handle)
{
  this->flg_safe_delete_ = 0;

  if (this->stream_.open (*this,
                           handle,
                           0, // completion key,
                           this->task().proactor(),
                           1 ) == -1)
    {
      ACE_ERROR ((LM_ERROR,
                ACE_TEXT ("%%s p\n"),
                this->get_name(),
                ACE_TEXT ("ACE_SSL_Asynch_Stream::open")));

      this->stream_.close();
    }
}


int
SSLSession::start_asynch_read (ACE_Message_Block &mb, size_t bytes)
{
    return this->stream_.read (mb, bytes);
}
int
SSLSession::start_asynch_write (ACE_Message_Block &mb, size_t bytes)
{
    return this->stream_.write (mb, bytes);
}

void
SSLSession::handle_wakeup ()
{
  {
    ACE_GUARD (ACE_SYNCH_MUTEX, monitor, this->mutex() );

    this->flg_safe_delete_ = 1;
      
    if (!this->is_safe_to_delete())
      return;
  }
  this->manager()->destroy_session(this);
}
#endif  /* ACE_WIN32 && !ACE_HAS_WINCE || ACE_HAS_AIO_CALLS */
