// Cached_Allocator_Test.cpp,v 4.7 2002/08/23 22:34:13 shuston Exp

//
// ============================================================================

#include "test_config.h"
#include "ace/Malloc_T.h"
#include "ace/POSIX_AIO_Allocator.h"
#include "ace/Containers_T.h"

ACE_RCSID(tests, Cached_Allocator_Test, "Cached_Allocator_Test.cpp,v 4.7 2002/08/23 22:34:13 shuston Exp")


// **************************************************************
static u_long g_num =0;

class Data
{
public :

  static void   free (Data * ptr);

  static Data * create(ACE_Allocator * allocator, int flg_big);

  Data (ACE_Allocator * allocator=0)
    : data_     ("BaseData"), 
      num_      (++g_num),
      allocator_(allocator),
      next_     (0),
      prev_     (0)
    {;}

  virtual ~Data(){;}

  const char * data_;
  u_long num_;

  virtual void  print()
    {
      dump (sizeof(Data),"-d");
    }

  void  dump(unsigned int sz, const char * txt)
    {
      ACE_DEBUG((LM_DEBUG,  
        "%s sz=%u this=%@ N->=%@ P<-=%@ al=%@ n=%u\n",
        txt,
        (unsigned int) sz,
        this,
        this->next_,
        this->prev_,
        this->allocator_,
        (unsigned int) num_));
    }

  ACE_Allocator * allocator_;

  Data * next_;
  Data * prev_;
};

class BigData : public Data
{
public :
  BigData (ACE_Allocator * allocator)
    : Data(allocator)
    { 
      data_="BigData";
    }

  virtual void  print()
    {
      dump (sizeof(BigData),"+D");
    }

  char fill_ [17];
};

void Data::free(Data * ptr)
{
  if (ptr == 0)
    return;

  ACE_Allocator * allocator = ptr->allocator_;

  if (allocator == 0)
    {
      delete ptr;
      return;
    }

  ptr->~Data ();

  allocator->free (ACE_static_cast (void*,ptr));

  return;
}

Data *
Data::create (ACE_Allocator * allocator, int flg_big)
{
  void * chunk =0;

  if ( allocator != 0)
    chunk = allocator->malloc (sizeof(BigData));
  else
    chunk = ACE_OS::malloc (sizeof(BigData));

  Data * result =0;

  if (flg_big)
    ACE_NEW_MALLOC_RETURN 
     (result,
      ACE_static_cast (BigData *, chunk),
      BigData(allocator),
      0);
  else
    ACE_NEW_MALLOC_RETURN 
     (result,
      ACE_static_cast (Data *, chunk),
      Data(allocator),
      0);

  return result;
}


// **************************************************************
//  Results Collection and Iterator
// **************************************************************
class Data_List : public ACE_Double_Linked_List<Data>
{
public:
  Data_List (void);
  virtual ~Data_List (void);
};

class Data_List_Iterator :  public ACE_Double_Linked_List_Iterator<Data>
{
public:
  Data_List_Iterator (const Data_List & list);
  virtual ~Data_List_Iterator (void);

  Data * get_current_and_advance ();
};
  
Data_List::Data_List (void)
{
}

Data_List::~Data_List (void)
{
  Data_List_Iterator itr( *this);

  for (;;)
   {
      Data *  result = itr.get_current_and_advance ();
      
      if (result == 0)
        break;
      
      Data::free(result);
  }
}

Data_List_Iterator::Data_List_Iterator (const Data_List & list)
  : ACE_Double_Linked_List_Iterator<Data> (list)
{
}

Data_List_Iterator::~Data_List_Iterator (void)
{
}

Data * 
Data_List_Iterator::get_current_and_advance ()
{
  Data * result = this->next ();

  this->advance();

  return result;
}

// ********************************************************************

typedef ACE_Cached_List_Allocator<BigData,ACE_SYNCH_NULL_MUTEX> Data_Allocator;


int
ACE_TMAIN (int argc, ACE_TCHAR *argv[])
{
  ACE_START_TEST (ACE_TEXT ("Cached_Allocator_Test"));

  ACE_UNUSED_ARG(argc);
  ACE_UNUSED_ARG(argv);

  int i=0;


  ACE_DEBUG ((LM_INFO,
              ACE_TEXT ("Creating allocator: ")
              ACE_TEXT ("sizeof BigData=%u u_long=%u\n"),
              sizeof(BigData),
              sizeof(u_long)));
  
  Data_Allocator allocator ;

  Data_List list;

  Data_List_Iterator itr(list);

  Data *  result =0;


  ACE_DEBUG ((LM_INFO,
              ACE_TEXT ("Fill list...\n")));

  for ( i=0; i < 12; ++ i)
    {
      ACE_Allocator * al = (i<6) ? &allocator :0;
      result = Data::create(al , i%2);

      list.insert_tail (result);
    }

  ACE_DEBUG ((LM_INFO,
              ACE_TEXT ("Iterate...\n")));


  itr.first();
  for (i=0;  ;++i)
   {
      result = itr.get_current_and_advance ();
      
      if (result == 0)
        break;
      
      result->print();

      ACE_Allocator * al = result->allocator_;
      
      list.remove(result);
      Data::free(result);

      result = Data::create(al, (i+1)%2);

      list.insert_head (result);

      result = Data::create(al, (i)%2);

      list.insert_head (result);
  }

  result = itr.get_current_and_advance ();
  ACE_ASSERT ( result==0);


  
  ACE_DEBUG ((LM_INFO,
              ACE_TEXT ("Removing all...\n")));

  itr.first();
  for (;;)
   {
      result = itr.get_current_and_advance ();
      
      if (result == 0)
        break;

      result->print();
      list.remove(result);

      Data::free(result);
  }

  result = itr.get_current_and_advance ();
  ACE_ASSERT ( result==0);

  ACE_END_TEST;
  return 0;
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)

template class ACE_Cached_Mem_Pool_Node<Data>;
template class ACE_Free_List<ACE_Cached_Mem_Pool_Node<Data> >;
template class ACE_Locked_Free_List<ACE_Cached_Mem_Pool_Node<Data>, ACE_SYNCH_NULL_MUTEX>;
template class ACE_Cached_List_Allocator<Data, ACE_SYNCH_NULL_MUTEX>;

template class ACE_Double_Linked_List<Data>;
template class ACE_Double_Linked_List_Iterator_Base<Data>;
template class ACE_Double_Linked_List_Iterator<Data>;

#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
#error  !!!!!!
#pragma instantiate ACE_Free_List<ACE_Cached_Mem_Pool_Node<Data> >

#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
