/*********************************************************************
** Copyright (C) 2003 Terabit Pty Ltd.  All rights reserved.
**
** This file is part of the POSIX-Proactor module.
**
** This file may be distributed and used only under the terms of the  
** Terabit Public License as defined by Terabit Pty Ltd of Australia   
** and appearing in the file tlicense.txt included in the packaging of
** this module and available at http://www.terabit.com.au/license.php.
**
** Contact support@terabit.com.au if any conditions of this licensing
** are not clear to you.
**********************************************************************/
// ============================================================================
/**
 *  @file ProactorTask.cpp
 *
 *
 *  @author Alexander Libman <libman@terabit.com.au>
 */
// ============================================================================

#include "ProactorTask.h"

#if defined (ACE_HAS_THREADS) 

#if (defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)) || defined(ACE_HAS_AIO_CALLS)

#if defined (ACE_WIN32) 

#  include "ace/WIN32_Proactor.h"

#else  /* defined (ACE_WIN32) */

#  include "ace/POSIX_Proactor.h"
#  include "ace/POSIX_CB_Proactor.h"
#  include "ace/SUN_Proactor.h"

#endif /* defined (ACE_WIN32) */

// *****************************************************************
//
// *****************************************************************

ProactorTask::ProactorTask (TestCfg & cfg)
  : TaskPool (cfg),
    proactor_ (0)
{
}

ProactorTask::~ProactorTask (void)
{
  this->stop_and_free();
}

int
ProactorTask::run_event_loop(void)
{
  int rc = -1;

  if (this->proactor_ != 0)
    rc = this->proactor_->proactor_run_event_loop ();
  
  return rc;
}

int
ProactorTask::end_event_loop(void)
{
  int rc = -1;

  ACE_DEBUG ((LM_DEBUG,
                  ACE_TEXT ("(%t) Calling End Proactor event loop\n")));

  if (this->proactor_ != 0)
    rc = this->proactor_->proactor_end_event_loop ();

  return rc;
}


int
ProactorTask::create_demultiplexor(void)
{
  ACE_ASSERT (this->proactor_ == 0);

  const ACE_TCHAR * str_type = ACE_TEXT("DEFAULT");


#if defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)

  ACE_WIN32_Proactor * proactor_impl = 0;

  str_type = ACE_TEXT("WIN32");

  ACE_NEW_RETURN (proactor_impl,
                  ACE_WIN32_Proactor,
                  -1);

  
  
#elif defined (ACE_HAS_AIO_CALLS) || defined (ACE_HAS_AIO_EMULATION)

  ProactorType ptype = this->cfg_.proactor_type ();
  size_t max_op      = this->cfg_.max_aio_operations ();
  int  sig_num       = this->cfg_.sig_num();
  int  leader_flg    = this->cfg_.leader_type();
  int  start_aio_flg = this->cfg_.start_aio_type();

  ACE_POSIX_Proactor * proactor_impl = 0;

  switch (ptype)
    {
    case AIOCB:
      str_type = ACE_TEXT("AIOCB");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_POSIX_AIOCB_Proactor (max_op,
                                                sig_num,
                                                leader_flg,
                                                start_aio_flg),
                      -1);
      break;

    case SIG:
      str_type = ACE_TEXT("SIG");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_POSIX_SIG_Proactor (max_op,
                                              sig_num,
                                              leader_flg,
                                              start_aio_flg),
                      -1);
      break;

    case SUN:
      str_type = ACE_TEXT("SUN");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_SUN_Proactor (max_op,
                                        sig_num,
                                        leader_flg,
                                        start_aio_flg),
                      -1);
      break;
      
    case CB:
      str_type = ACE_TEXT("CB-SGI");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_POSIX_CB_Proactor (max_op),
                      -1);
      break;

    case SELECT:
      str_type = ACE_TEXT("SELECT");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_Select_Proactor (max_op,
                                          sig_num,
                                          leader_flg,
                                          start_aio_flg),
                      -1);
      break;

    case POLL:
      str_type = ACE_TEXT("POLL");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_Poll_Proactor (max_op,
                                          sig_num,
                                          leader_flg,
                                          start_aio_flg),
                      -1);
      break;

    case LINUXRT:
      str_type = ACE_TEXT("LINUX_RT");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_Linux_RT_Proactor (max_op,
                                          sig_num,
                                          1,
                                          start_aio_flg),
                      -1);
      break;

    case EPOLL:
      str_type = ACE_TEXT("EPOLL");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_Event_Poll_Proactor (max_op,
                                          sig_num,
                                          leader_flg,
                                          start_aio_flg),
                      -1);
      break;
    case DEVPOLL:
      str_type = ACE_TEXT("DEVPOLL");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_Dev_Poll_Proactor (max_op,
                                          sig_num,
                                          leader_flg,
                                          start_aio_flg),
                      -1);
      break;


    case LINUX:
      str_type = ACE_TEXT("LINUX");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_LINUX_Proactor (max_op,
                                          sig_num,
                                          leader_flg,
                                          start_aio_flg),
                      -1);
      break;
      
    case DUMMY:
      str_type = ACE_TEXT("DUMMY");
      ACE_NEW_RETURN (proactor_impl,
                      ACE_Dummy_Proactor (),
                      -1);
    default:
      break;
  }

#endif // (ACE_WIN32) && !defined (ACE_HAS_WINCE)

  ACE_DEBUG ((LM_DEBUG,
              ACE_TEXT("(%t) Create Proactor Type = %s\n"),
              str_type));

  // always delete implementation  1 , not  !(proactor_impl == 0)
  ACE_NEW_RETURN (this->proactor_,
                  ACE_Proactor (proactor_impl, 1 ), 
                  -1);
  
  return 0;
}

int
ProactorTask::delete_demultiplexor (void)
{
  ACE_DEBUG ((LM_DEBUG,
              ACE_TEXT ("(%t) Delete Proactor\n")));

  //ACE_Proactor::close_singleton ();
  delete this->proactor_;
  this->proactor_ = 0;

  return 0;
}

#endif // (defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)) || defined(ACE_HAS_AIO_CALLS)

#endif // ACE_HAS_THREADS 
