// Proactor_Test.cpp,v 1.42 2002/11/27 01:09:10 shuston Exp

// ============================================================================
/**
 *  @file X_Test.cpp
 *
 *  X_Test.cpp,v 1.42 2002/11/27 01:09:10 shuston Exp
 *
 *  This program illustrates how the ACE_Proactor can be used to
 *  implement an application that does various asynchronous
 *  operations.
 *
 *  @author Alexander Libman <alibman@optusnet.com.au>
 */
// ============================================================================

#include "test_config.h"

ACE_RCSID (tests,
           X_Test,
           "X_Test.cpp,v 1.42 2002/11/27 01:09:10 shuston Exp")


#include "ace/Signal.h"

#include "ace/Service_Config.h"
#include "ace/INET_Addr.h"
#include "ace/Get_Opt.h"
#include "ace/streams.h"

#include "ace/Proactor.h"
#include "ace/Asynch_Acceptor.h"
#include "ace/Asynch_Connector.h"
#include "ace/Task.h"

#if defined (ACE_WIN32) && !defined (ACE_HAS_WINCE)

#  include "ace/WIN32_Proactor.h"

#elif defined (ACE_HAS_AIO_CALLS)

#  include "ace/POSIX_Proactor.h"
#  include "ace/POSIX_CB_Proactor.h"
#  include "ace/SUN_Proactor.h"

#endif /* defined (ACE_WIN32) && !defined (ACE_HAS_WINCE) */



class LogLocker
{
public:

  LogLocker () { ACE_LOG_MSG->acquire (); }
  virtual ~LogLocker () { ACE_LOG_MSG->release (); }
};



// Function to remove signals from the signal mask.
static int
disable_signal (int sigmin, int sigmax)
{
#ifndef ACE_WIN32

  sigset_t signal_set;
  if (sigemptyset (&signal_set) == - 1)
    ACE_ERROR ((LM_ERROR,
                ACE_TEXT ("Error: (%P|%t):%p\n"),
                ACE_TEXT ("sigemptyset failed")));

  for (int i = sigmin; i <= sigmax; i++)
    sigaddset (&signal_set, i);

  //  Put the <signal_set>.
  if (ACE_OS::pthread_sigmask (SIG_BLOCK, &signal_set, 0) != 0)
    ACE_ERROR ((LM_ERROR,
                ACE_TEXT ("Error: (%P|%t):%p\n"),
                ACE_TEXT ("pthread_sigmask failed")));
#else
  ACE_UNUSED_ARG (sigmin);
  ACE_UNUSED_ARG (sigmax);
#endif /* ACE_WIN32 */

  return 1;
}



// *************************************************************
//  MyTask is ACE_Task resposible for :
//  1. creation and deletion of
//     Proactor and Proactor thread pool
//  2. running Proactor event loop
// *************************************************************
class MyTask : public ACE_Task<ACE_MT_SYNCH>
{
public:
  MyTask (ACE_Thread_Manager * mgr);
  
  virtual ~MyTask() ;
   
  virtual int svc (void);

  int start (int num_threads);
  
  int stop  (void);

  
private:
  
  ACE_SYNCH_RECURSIVE_MUTEX lock_;
  ACE_Thread_Semaphore sem_;
  
};

MyTask::MyTask (ACE_Thread_Manager * mgr)
   : ACE_Task<ACE_MT_SYNCH> (mgr),
     lock_ (),
     sem_  (0)
{
}

MyTask::~MyTask()
{
  //this->stop();
}

int
MyTask::start (int num_threads)
{

  if (this->activate (THR_NEW_LWP|THR_JOINABLE, num_threads) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       ACE_TEXT ("%p.\n"),
                       ACE_TEXT ("unable to activate thread pool")),
                      -1);

                      
  for (; num_threads > 0; num_threads--)
    {
      sem_.acquire ();
    }

  return 0;
}


int
MyTask::stop ()
{
  //if (this->thr_count() == 0)
  //   return 0;


  if (this->wait () == -1)
    ACE_ERROR ((LM_ERROR,
                ACE_TEXT ("%p.\n"),
                ACE_TEXT ("unable to stop thread pool")));

  return 0;
}



int
MyTask::svc (void)
{
  ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("(%t) MyTask started\n")));

  disable_signal (ACE_SIGRTMIN, ACE_SIGRTMAX);

  // signal that we are ready
  sem_.release (1);

  ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("(%t) MyTask finished\n")));
  return 0;
}


int
ACE_TMAIN (int argc, ACE_TCHAR *argv[])
{
  //ACE_START_TEST (ACE_TEXT ("X_Test"));


  
  

  ACE_Thread_Manager mgr;
  MyTask    task1(&mgr);  

  if (task1.start (1) == 0)
    {
       ACE_Time_Value  sleep_time (1);
           
       while ( sleep_time != ACE_Time_Value::zero)
            {
              ACE_Countdown_Time countdown ( & sleep_time );

              ACE_OS::sleep (sleep_time );
            }
      
    }


  
   
  ACE_DEBUG ((LM_DEBUG,
              ACE_TEXT ("(%t) Stop Thread Pool Task\n")));

  task1.stop ();
 
  
//  ACE_END_TEST;

  return 0;
}

